# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'OS'
        db.create_table(u'report_os', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('major', self.gf('django.db.models.fields.SmallIntegerField')()),
            ('minor', self.gf('django.db.models.fields.SmallIntegerField')()),
        ))
        db.send_create_signal(u'report', ['OS'])

        # Adding model 'Host'
        db.create_table(u'report_host', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('hostname', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('ssh_name', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('ssh_password', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('sudo_password', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('su_login', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('local', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('os', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.OS'], null=True, blank=True)),
        ))
        db.send_create_signal(u'report', ['Host'])

        # Adding model 'TestGroup'
        db.create_table(u'report_testgroup', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255, db_index=True)),
            ('xccdf_id', self.gf('django.db.models.fields.CharField')(max_length=128)),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.TestGroup'], null=True, blank=True)),
        ))
        db.send_create_signal(u'report', ['TestGroup'])

        # Adding model 'TestGroupResult'
        db.create_table(u'report_testgroupresult', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.TestGroup'])),
            ('result', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.Result'])),
            ('parent', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='direct_children_set', null=True, to=orm['report.TestGroupResult'])),
            ('root', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='all_children_set', null=True, to=orm['report.TestGroupResult'])),
            ('test_count', self.gf('django.db.models.fields.SmallIntegerField')(default=0, null=True, blank=True)),
            ('failed_test_count', self.gf('django.db.models.fields.SmallIntegerField')(default=0, null=True, blank=True)),
            ('ni_test_count', self.gf('django.db.models.fields.SmallIntegerField')(default=0, null=True, blank=True)),
            ('na_test_count', self.gf('django.db.models.fields.SmallIntegerField')(default=0, null=True, blank=True)),
        ))
        db.send_create_signal(u'report', ['TestGroupResult'])

        # Adding model 'Run'
        db.create_table(u'report_run', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('dt_submitted', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('dt_finished', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'report', ['Run'])

        # Adding model 'HostRun'
        db.create_table(u'report_hostrun', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('dt_finished', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('host', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.Host'])),
            ('run', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.Run'])),
            ('state', self.gf('django.db.models.fields.CharField')(default='r', max_length=1, db_index=True)),
            ('risk', self.gf('django.db.models.fields.CharField')(db_index=True, max_length=16, null=True, blank=True)),
        ))
        db.send_create_signal(u'report', ['HostRun'])

        # Adding model 'Result'
        db.create_table(u'report_result', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('dt_submitted', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('dt_finished', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('filename', self.gf('django.db.models.fields.CharField')(max_length=128, null=True, blank=True)),
            ('hostrun', self.gf('django.db.models.fields.related.OneToOneField')(to=orm['report.HostRun'], unique=True)),
            ('identity', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('hostname', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('test_count', self.gf('django.db.models.fields.SmallIntegerField')(null=True, blank=True)),
            ('failed_test_count', self.gf('django.db.models.fields.SmallIntegerField')(null=True, blank=True)),
            ('ni_test_count', self.gf('django.db.models.fields.SmallIntegerField')(null=True, blank=True)),
            ('na_test_count', self.gf('django.db.models.fields.SmallIntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'report', ['Result'])

        # Adding model 'Address'
        db.create_table(u'report_address', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('address', self.gf('django.db.models.fields.GenericIPAddressField')(max_length=39)),
            ('result', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.Result'])),
        ))
        db.send_create_signal(u'report', ['Address'])

        # Adding model 'Test'
        db.create_table(u'report_test', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255, db_index=True)),
            ('id_ref', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('description', self.gf('django.db.models.fields.TextField')()),
            ('component', self.gf('django.db.models.fields.CharField')(max_length=128, null=True, blank=True)),
            ('fix', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('fix_type', self.gf('django.db.models.fields.CharField')(max_length=32, null=True, blank=True)),
            ('fixtext', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.TestGroup'], null=True, blank=True)),
        ))
        db.send_create_signal(u'report', ['Test'])

        # Adding model 'TestResult'
        db.create_table(u'report_testresult', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('state', self.gf('django.db.models.fields.CharField')(max_length=3, db_index=True)),
            ('date', self.gf('django.db.models.fields.DateTimeField')()),
            ('test', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.Test'])),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(related_name='testresult_set', to=orm['report.TestGroupResult'])),
            ('root_group', self.gf('django.db.models.fields.related.ForeignKey')(related_name='all_tests', to=orm['report.TestGroupResult'])),
            ('result', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.Result'])),
        ))
        db.send_create_signal(u'report', ['TestResult'])

        # Adding model 'TestLog'
        db.create_table(u'report_testlog', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('message', self.gf('django.db.models.fields.TextField')()),
            ('level', self.gf('django.db.models.fields.CharField')(max_length=32)),
            ('date', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('result', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.TestResult'])),
        ))
        db.send_create_signal(u'report', ['TestLog'])

        # Adding model 'Risk'
        db.create_table(u'report_risk', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('message', self.gf('django.db.models.fields.TextField')()),
            ('level', self.gf('django.db.models.fields.CharField')(max_length=32)),
            ('result', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['report.TestResult'])),
        ))
        db.send_create_signal(u'report', ['Risk'])


    def backwards(self, orm):
        # Deleting model 'OS'
        db.delete_table(u'report_os')

        # Deleting model 'Host'
        db.delete_table(u'report_host')

        # Deleting model 'TestGroup'
        db.delete_table(u'report_testgroup')

        # Deleting model 'TestGroupResult'
        db.delete_table(u'report_testgroupresult')

        # Deleting model 'Run'
        db.delete_table(u'report_run')

        # Deleting model 'HostRun'
        db.delete_table(u'report_hostrun')

        # Deleting model 'Result'
        db.delete_table(u'report_result')

        # Deleting model 'Address'
        db.delete_table(u'report_address')

        # Deleting model 'Test'
        db.delete_table(u'report_test')

        # Deleting model 'TestResult'
        db.delete_table(u'report_testresult')

        # Deleting model 'TestLog'
        db.delete_table(u'report_testlog')

        # Deleting model 'Risk'
        db.delete_table(u'report_risk')


    models = {
        u'report.address': {
            'Meta': {'object_name': 'Address'},
            'address': ('django.db.models.fields.GenericIPAddressField', [], {'max_length': '39'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'result': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.Result']"})
        },
        u'report.host': {
            'Meta': {'object_name': 'Host'},
            'hostname': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'local': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'os': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.OS']", 'null': 'True', 'blank': 'True'}),
            'ssh_name': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'ssh_password': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'su_login': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'sudo_password': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'})
        },
        u'report.hostrun': {
            'Meta': {'ordering': "('-run__dt_submitted',)", 'object_name': 'HostRun'},
            'dt_finished': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'host': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.Host']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'risk': ('django.db.models.fields.CharField', [], {'db_index': 'True', 'max_length': '16', 'null': 'True', 'blank': 'True'}),
            'run': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.Run']"}),
            'state': ('django.db.models.fields.CharField', [], {'default': "'r'", 'max_length': '1', 'db_index': 'True'})
        },
        u'report.os': {
            'Meta': {'ordering': "('major', 'minor')", 'object_name': 'OS'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'major': ('django.db.models.fields.SmallIntegerField', [], {}),
            'minor': ('django.db.models.fields.SmallIntegerField', [], {})
        },
        u'report.result': {
            'Meta': {'object_name': 'Result'},
            'dt_finished': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'dt_submitted': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'failed_test_count': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'filename': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'hostname': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'hostrun': ('django.db.models.fields.related.OneToOneField', [], {'to': u"orm['report.HostRun']", 'unique': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identity': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'na_test_count': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'ni_test_count': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'test_count': ('django.db.models.fields.SmallIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'report.risk': {
            'Meta': {'object_name': 'Risk'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'result': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.TestResult']"})
        },
        u'report.run': {
            'Meta': {'ordering': "('-dt_submitted',)", 'object_name': 'Run'},
            'dt_finished': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            'dt_submitted': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'report.test': {
            'Meta': {'object_name': 'Test'},
            'component': ('django.db.models.fields.CharField', [], {'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'fix': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'fix_type': ('django.db.models.fields.CharField', [], {'max_length': '32', 'null': 'True', 'blank': 'True'}),
            'fixtext': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.TestGroup']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'id_ref': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'})
        },
        u'report.testgroup': {
            'Meta': {'ordering': "('title',)", 'object_name': 'TestGroup'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.TestGroup']", 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'db_index': 'True'}),
            'xccdf_id': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        u'report.testgroupresult': {
            'Meta': {'ordering': "('group',)", 'object_name': 'TestGroupResult'},
            'failed_test_count': ('django.db.models.fields.SmallIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.TestGroup']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'na_test_count': ('django.db.models.fields.SmallIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'ni_test_count': ('django.db.models.fields.SmallIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'}),
            'parent': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'direct_children_set'", 'null': 'True', 'to': u"orm['report.TestGroupResult']"}),
            'result': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.Result']"}),
            'root': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'all_children_set'", 'null': 'True', 'to': u"orm['report.TestGroupResult']"}),
            'test_count': ('django.db.models.fields.SmallIntegerField', [], {'default': '0', 'null': 'True', 'blank': 'True'})
        },
        u'report.testlog': {
            'Meta': {'object_name': 'TestLog'},
            'date': ('django.db.models.fields.DateTimeField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'result': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.TestResult']"})
        },
        u'report.testresult': {
            'Meta': {'ordering': "('state',)", 'object_name': 'TestResult'},
            'date': ('django.db.models.fields.DateTimeField', [], {}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'testresult_set'", 'to': u"orm['report.TestGroupResult']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'result': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.Result']"}),
            'root_group': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'all_tests'", 'to': u"orm['report.TestGroupResult']"}),
            'state': ('django.db.models.fields.CharField', [], {'max_length': '3', 'db_index': 'True'}),
            'test': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['report.Test']"})
        }
    }

    complete_apps = ['report']


# -*- coding: utf-8 -*-
from django.db.models import Q

from django import forms

from .models import Host, HostRun, TestResult


class NewHostForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(NewHostForm, self).__init__(*args, **kwargs)
        self.fields['ssh_name'].label = "SSH Login"
        self.fields['ssh_password'].label = "SSH Password"
        self.fields['sudo_password'].label = "sudo Password"
        self.fields['su_login'].label = "su Login"
        self.fields['local'].label = "Local scan"

    class Meta:
        model = Host


class NewRunForm(forms.Form):

    hosts = forms.ModelMultipleChoiceField(queryset=Host.objects.all())
    #tests = forms.ModelMultipleChoiceField(queryset=TestGroup.objects.all())

    def __init__(self, *args, **kwargs):
        super(NewRunForm, self).__init__(*args, **kwargs)
        self.fields['hosts'].label = "Hosts"


class FilterForm(forms.Form):
    """
    form displayed in the toolbar, you can filter runs by host, risk level or just plain search
    """
    search = forms.CharField(required=False, widget=forms.TextInput(
        attrs={
            'id': 'global-search',
            'placeholder': "Search...",
            'class': 'form-control',
        })
    )
    hosts = forms.MultipleChoiceField(required=False, widget=forms.SelectMultiple(
        attrs={
            'class': 'selectpicker global-filter host-filter',
            'data-live-search': 'true',
        })
    )
    risk = forms.ChoiceField(required=False, widget=forms.Select(
        attrs={
            'class': 'selectpicker global-filter risk-filter',
        })
    )


    def __init__(self, *args, **kwargs):
        super(FilterForm, self).__init__(*args, **kwargs)
        risk_choices = []
        for risk in HostRun.objects.risks():
            if risk and risk not in risk_choices:
                risk_choices.append(risk)
        risk_choices = zip(risk_choices, [risk.capitalize() for risk in risk_choices])
        # first option searches in every release
        risk_choices.insert(0, ('', 'All Risk Levels'))
        self.fields['risk'].choices = risk_choices

        host_choices = [(hr['host__hostname'], hr['host__hostname'])
            for hr in HostRun.objects.values('host__hostname').order_by('host__hostname').distinct()
        ]
        # first option searches in every release
        host_choices.insert(0, ('', 'All Hosts'))
        self.fields['hosts'].choices = host_choices

    def is_valid(self):
        valid = super(FilterForm, self).is_valid()
        if not valid:
            return valid
        if not self.cleaned_data['hosts'] and not self.cleaned_data['risk'] and not self.cleaned_data['search']:
            return False
        return True

class StateFilterForm(forms.Form):
    """ filter test results by their state """

    def __init__(self, *args, **kwargs):
        result = kwargs.pop('result')
        super(StateFilterForm, self).__init__(*args, **kwargs)

        states = result.states()

        #states.insert(0, ('', 'All'))

        self.blacklisted_states = ['notchecked']

        sorted_mapping = sorted(TestResult.TEST_STATES.get_mapping(), key=lambda x: x[0])

        for state_key, state_css in sorted_mapping:
            try:
                print_state = states[state_css]
            except KeyError:
                if not TestResult.TEST_STATES[state_key] in self.blacklisted_states:
                    print_state = "%s (0)" % TestResult.TEST_STATES.display(state_key)
                else:
                    continue
            field_name = state_css + str(result.id)
            self.fields[field_name] = forms.BooleanField(label=print_state, required=False)
        #self.fields[field_name].widget.attrs['class'] = "state-filter"
        #self.fields[field_name].widget.attrs['multiple'] = "multiple"
        #try:
        #    initial = args[0]
        #except IndexError:
        #    pass
        #else:
        #    self.fields[field_name].initial = initial

    def all_checked(self):
        return len(self.initial) >= len(TestResult.TEST_STATES.get_mapping()) - len(self.blacklisted_states)


class ListActionForm(forms.Form):
    ACTIONS = {
        'delete_selected': 'Delete selected',
    }
    runs    = forms.MultipleChoiceField()
    action  = forms.ChoiceField(choices=ACTIONS.items())
    confirm = forms.BooleanField(required=False)


class DeleteOlderForm(forms.Form):
    host = forms.ChoiceField(required=False)

    def __init__(self, *args, **kwargs):
        super(DeleteOlderForm, self).__init__(*args, **kwargs)
        host_choices = [(hr['host__hostname'], hr['host__hostname'])
            for hr in HostRun.objects.values('host__hostname').order_by('host__hostname').distinct()
        ]
        self.fields['host'].choices = host_choices





from __future__ import unicode_literals
import unittest
import shutil
import os

from preupg import script_api
from preupg.utils import FileHelper
from preupg import settings

try:
    import base
except ImportError:
    import tests.base as base


class TestAPICheck(base.TestCase):

    dirname = os.path.join(os.path.dirname(__file__), 'tmp')
    solution_txt = settings.solution_txt
    api_files = "api_files"
    dist_native = 'dist_native'

    def setUp(self):
        if not os.path.isdir(os.path.join(self.dirname, 'kickstart')):
            os.makedirs(os.path.join(self.dirname, 'kickstart'))
        script_api.VALUE_RPM_RHSIGNED = os.path.join(os.path.dirname(__file__), self.api_files, 'rpm_rhsigned')
        script_api.VALUE_RPM_QA = os.path.join(os.path.dirname(__file__), self.api_files, 'rpm_qa')
        script_api.VALUE_CHKCONFIG = os.path.join(os.path.dirname(__file__), self.api_files, 'chkconfig')
        script_api.VALUE_CONFIGCHANGED = os.path.join(os.path.dirname(__file__), self.api_files, 'rpm_etc_Va')
        script_api.VALUE_EXECUTABLES = os.path.join(os.path.dirname(__file__), self.api_files, 'executable')
        script_api.PREUPGRADE_CACHE = self.dirname
        script_api.SOLUTION_FILE = self.solution_txt
        if os.environ['XCCDF_VALUE_TMP_PREUPGRADE'] == "":
            script_api.VALUE_TMP_PREUPGRADE = self.dirname

    def test_solution_file(self):
        expected_output = ["Testing message"]
        script_api.solution_file('\n'.join(expected_output))
        output = FileHelper.get_file_content(os.path.join(script_api.VALUE_CURRENT_DIRECTORY, self.solution_txt),
                                             "r",
                                             method=True)
        self.assertEqual(expected_output, output)
        os.unlink(os.path.join(script_api.VALUE_CURRENT_DIRECTORY, self.solution_txt))

    def test_is_pkg_installed(self):
        expected_installed_pkg = "preupgrade-assistant"
        self.assertFalse(script_api.is_pkg_installed("preupgrade-assistant-modules"))
        self.assertTrue(script_api.is_pkg_installed(expected_installed_pkg))

    def test_check_applies_to(self):
        expected_rpms = "foobar,testbar"
        self.assertEqual(script_api.check_applies_to(expected_rpms), 0)

    def test_check_rpm_to(self):
        expected_rpms = "foobar,testbar"
        self.assertEqual(script_api.check_rpm_to(check_rpm=expected_rpms), 0)

    def test_not_check_rpm_to(self):
        expected_rpms = "ffoobar,testbar"
        try:
            self.assertEqual(script_api.check_rpm_to(check_rpm=expected_rpms), 0)
            self.assertTrue(False)
        except SystemExit:
            self.assertTrue(True)

    def test_check_rpm_to_binaries(self):
        expected_binaries = "strings,nm"
        self.assertEqual(script_api.check_rpm_to(check_bin=expected_binaries), 0)

    def test_not_check_rpm_to_binaries(self):
        expected_binaries = "/usr/bin/fooupg,/bin/preupg"
        try:
            self.assertEqual(script_api.check_rpm_to(check_bin=expected_binaries), 0)
            self.assertTrue(False)
        except SystemExit:
            self.assertTrue(True)

    def test_service_is_enabled(self):
        expected_service_enabled = "foo"
        expected_service_disabled = "foonetwork"
        self.assertTrue(script_api.service_is_enabled(expected_service_enabled))
        self.assertFalse(script_api.service_is_enabled(expected_service_disabled))

    def test_config_file_changed(self):
        self.assertTrue(script_api.config_file_changed("/etc/foo/test.conf"))
        self.assertFalse(script_api.config_file_changed("/etc/foobar/test.conf"))

    def test_is_dist_native(self):
        self.assertTrue(script_api.is_dist_native('foobar'))
        script_api.DEVEL_MODE = 1
        script_api.DIST_NATIVE = "all"
        self.assertTrue(script_api.is_dist_native("preupgrade-assistant"))
        script_api.DIST_NATIVE = "sign"
        self.assertFalse(script_api.is_dist_native("preupgrade-assistant"))
        script_api.DIST_NATIVE = os.path.join(os.path.dirname(__file__), self.api_files, self.dist_native)
        self.assertFalse(script_api.is_dist_native("non-sense"))

    def test_get_dist_native_list(self):
        expected_list = ['foobar',
                         'barfoo',
                         'testbar',
                         'footest']
        self.assertEqual(script_api.get_dist_native_list(), expected_list)

    def test_add_pkg_to_kickstart(self):
        expected_list = ['my_foo_pkg', 'my_bar_pkg']
        script_api.add_pkg_to_kickstart(['my_foo_pkg', 'my_bar_pkg'])
        for pkg in FileHelper.get_file_content(script_api.SPECIAL_PKG_LIST, 'rb', method=True):
            self.assertTrue(pkg.strip() in expected_list)
        script_api.add_pkg_to_kickstart('my_foo_pkg my_bar_pkg')
        for pkg in FileHelper.get_file_content(script_api.SPECIAL_PKG_LIST,'rb', method=True):
            self.assertTrue(pkg.strip() in expected_list)

    def test_deploy_hook_postupgrade(self):
        hook_dir = os.path.join(script_api.VALUE_TMP_PREUPGRADE, "hooks")
        if os.path.isdir(hook_dir):
            shutil.rmtree(hook_dir)
        deploy_type = "postupgrade"
        script_api.deploy_hook(deploy_type, "setup.py", "common.sh", "/usr/bin/nm")
        postupgrade_hook_dir = os.path.join(script_api.VALUE_TMP_PREUPGRADE,
                                            "hooks",
                                            script_api.MODULE_PATH,
                                            deploy_type,
                                            )
        for f in ["run_hook", "common.sh", "nm"]:
            self.assertTrue(os.path.isfile(os.path.join(postupgrade_hook_dir, f)))

    def test_deploy_hook_preupgrade(self):
        hook_dir = os.path.join(script_api.VALUE_TMP_PREUPGRADE, "hooks")
        if os.path.isdir(hook_dir):
            shutil.rmtree(hook_dir)
        deploy_type = "preupgrade"
        script_api.deploy_hook(deploy_type, "setup.py", "common.sh", "/usr/bin/nm")
        preupgrade_hook_dir = os.path.join(script_api.VALUE_TMP_PREUPGRADE,
                                            "hooks",
                                            script_api.MODULE_PATH,
                                            deploy_type,
                                            )
        for f in ["run_hook", "common.sh", "nm"]:
            self.assertTrue(os.path.isfile(os.path.join(preupgrade_hook_dir, f)))


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestAPICheck))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



import optparse


class CLIKickstart(object):
    """ Class for processing data from commandline """

    def __init__(self, args=None):
        """ parse arguments """
        self.parser = optparse.OptionParser()

        self.add_args()
        if args:
            self.opts, self.args = self.parser.parse_args(args=args)
        else:
            self.opts, self.args = self.parser.parse_args()

    def add_args(self):
        self.parser.add_option(
            "-y",
            "--assumeyes",
            action="store_true",
            default=False,
            help="Suppress user interaction"
        )


if __name__ == '__main__':
    x = CLIKickstart()
    print (x.args.id)



# -*- coding: utf-8 -*-
""" Database schema """
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.db.models.aggregates import Count
from django.http.response import Http404
import os
import datetime

from django.db import models
from django.conf import settings
from preupg.ui.config.models import AppSettings
from preupg.ui.utils.enum import Enum
from shutil import rmtree


class OS(models.Model):
    RHEL5 = 5
    RHEL6 = 6
    RHEL7 = 7

    MAJOR_VERSIONS = Enum([
        (RHEL5, 'RHEL-5', 'Red Hat Enterprise Linux 5'),
        (RHEL6, 'RHEL-6', 'Red Hat Enterprise Linux 6'),
        (RHEL7, 'RHEL-7', 'Red Hat Enterprise Linux 7'),
    ])
    major = models.SmallIntegerField(choices=MAJOR_VERSIONS.get_mapping())
    minor = models.SmallIntegerField()

    class Meta:
        ordering = ('major', 'minor')

    def __unicode__(self):
        return u"%s.%d" % (self.MAJOR_VERSIONS.get_help(self.major),
                           self.minor)


class Host(models.Model):
    """
    host where analysis was executed
    creds are required for distributive execution
    """
    hostname = models.CharField(max_length=255)
    ssh_name = models.CharField(max_length=255, blank=True, null=True)
    # TODO: security \/
    ssh_password = models.CharField(max_length=255, blank=True, null=True)
    sudo_password = models.CharField(max_length=255, blank=True, null=True)
    su_login = models.CharField(max_length=255, blank=True, null=True,
                                help_text="Alternative user")
    local = models.BooleanField(default=False, help_text="Local analysis")

    os = models.ForeignKey(OS, blank=True, null=True)

    class Meta:
        ordering = ('hostname',)

    def __unicode__(self):
        return u"%s (%s)" % (self.hostname, self.os)

    @classmethod
    def localhost(cls):
        return cls.objects.get(hostname="localhost")


class TestGroupMixin(object):
    def create_or_update(self, k):
        """
        return Test object for specified id_ref rule
        """
        try:
            tg = self.get(xccdf_id=k['xccdf_id'])
        except ObjectDoesNotExist:
            tg = TestGroup(**k)
            tg.save()
        else:
            self.filter(pk=tg.pk).update(**k)
        return tg

    def root(self):
        return self.filter(parent__isnull=True)


class TestGroupQuerySet(models.query.QuerySet, TestGroupMixin):
    pass


class TestGroupManager(models.Manager, TestGroupMixin):
    def get_query_set(self):
        return TestGroupQuerySet(self.model, using=self._db)


class TestGroup(models.Model):
    """
    Tests are composed into groups, user may pick which groups to choose
    """
    title = models.CharField(max_length=255, db_index=True)
    xccdf_id = models.CharField(max_length=128)
    parent = models.ForeignKey('self', blank=True, null=True)

    objects = TestGroupManager()

    class Meta:
        ordering = ('title', )

    def __unicode__(self):
        if self.parent:
            return u"%s [%s]" % (self.title, self.parent)
        else:
            return u"%s" % (self.title)

    def has_children(self):
        return TestGroup.objects.filter(parent=self).exists()

    def children(self):
        return TestGroup.objects.filter(parent=self)

    def is_root(self):
        """ is this group root group? == does it have parent? """
        return self.parent is None


class TestGroupResultMixin(object):
    def for_result(self, result):
        return self.filter(result=result)

    def search_in_title(self, query_string):
        return self.filter(group__title__icontains=query_string)

    def root(self):
        return self.filter(parent__isnull=True)


class TestGroupResultQuerySet(models.query.QuerySet, TestGroupResultMixin):
    pass


class TestGroupResultManager(models.Manager, TestGroupResultMixin):
    def get_query_set(self):
        return TestGroupResultQuerySet(self.model, using=self._db)


class TestGroupResult(models.Model):
    """
    Result of test group -- combine run and testgroup together
    """
    group = models.ForeignKey(TestGroup)
    result = models.ForeignKey('Result')
    parent = models.ForeignKey('self', related_name="direct_children_set", blank=True, null=True)
    # this should never be None, but if this group is root also, it has to be saved first
    root = models.ForeignKey('self', related_name="all_children_set", blank=True, null=True)

    test_count = models.SmallIntegerField(blank=True, null=True, default=0)
    failed_test_count = models.SmallIntegerField(blank=True, null=True, default=0)
    ni_test_count = models.SmallIntegerField(blank=True, null=True, default=0)
    na_test_count = models.SmallIntegerField(blank=True, null=True, default=0)

    objects = TestGroupResultManager()

    class Meta:
        ordering = ('group', )

    def __unicode__(self):
        return u"%s %s" % (self.group, self.result)

    def has_children(self):
        return TestGroupResult.objects.filter(parent=self).exists()

    def children(self):
        return TestGroupResult.objects.filter(parent=self)

    def testresults(self):
        return self.testresult_set.all()

    def get_root(self):
        """ return root group -- top-most parent """
        if self.root is not None:
            return self.root
        group = self
        parent = self.parent
        while parent is not None:
            group = parent
            parent = group.parent
        self.root = group
        self.save()
        return group

    #def title_matches_filter(self, search_string):
    #    """ does this group matches searching? """
    #    return search_string.lower() in self.group.title


class RunMixin(object):

    def create_for_host(self, host):
        run = Run()
        run.save()
        HostRun.objects.create(host=host, run=run)
        return run

    def bulk_create_for_run(self, hosts):
        """
        create hostruns for list of hosts specified as Query in variable hosts
        """
        run = Run()
        run.save()
        hostrun_list = []
        for host in hosts:
            hostrun_list.append(HostRun(host=host, run=run))

        HostRun.objects.bulk_create(hostrun_list)
        return run


class RunQuerySet(models.query.QuerySet, RunMixin):
    pass


class RunManager(models.Manager, RunMixin):
    def get_query_set(self):
        return RunQuerySet(self.model, using=self._db)


class Run(models.Model):
    """
    one run may consist of several results (from multiple hosts)
    """
    dt_submitted = models.DateTimeField(auto_now_add=True)
    dt_finished = models.DateTimeField(blank=True, null=True)
    objects = RunManager()

    class Meta:
        ordering = ('-dt_submitted', )

    def __unicode__(self):
        if not self.dt_finished:
            return u"#%d %s (running)" % (self.id, self.dt_submitted)
        else:
            return u"#%d %s--%s" % (self.id, self.dt_submitted,
                                    self.dt_finished)

    def results(self):
        """ Return all Results from this run"""
        return Result.objects.filter(hostrun__run=self)

    def testresults(self):
        """ Return all TestResults from this run"""
        return TestResult.objects.filter(group__result__hostrun__run=self)

    def get_run_dir(self):
        results_dir = os.path.abspath(settings.RESULTS_DIR)
        path = os.path.join(results_dir, str(self.id))
        if not os.path.isdir(path):
            os.makedirs(path, mode=0o0755)
        return path

    def has_local(self):
        """ should there be some scan performed locally? """
        return self.hostrun_set.filter(host__local=True).exists()

    def local(self):
        """ return local hostrun assigned to this run """
        return HostRun.objects.get(run=self, host__local=True)

    def remotes(self):
        """ Return remote HostRuns """
        return HostRun.objects.filter(run=self, host__local=False)

    def all_done(self):
        """ if there are no running tasks, everything is done """
        return not HostRun.objects.for_run(self).running().exists()

    def finish(self):
        self.dt_finished = datetime.datetime.now()
        self.save(update_fields=["dt_finished"])

    def first_hostrun(self):
        return self.hostrun_set.all()[0]


class HostRunMixin(object):
    def for_run(self, run):
        return self.filter(run=run)

    def for_result(self, result_id):
        result = self.filter(result=result_id)
        if not result:
            raise Http404('There is no such run.')
        else:
            return result

    def running(self):
        return self.filter(state=HostRun.RUNNING)

    def finished(self):
        return self.filter(state=HostRun.FINISHED)

    def hosts(self):
        """ return list of hostname strings found in hostruns """
        return self.values_list("result__hostname", flat=True)

    def risks(self):
        """ return list of risk strings found in hostruns """
        return self.values_list("risk", flat=True).distinct()

    def by_risk(self, risk):
        return self.filter(risk=risk)

    def by_hosts_processed(self, hosts):
        return self.filter(result__hostname__in=hosts)


class HostRunQuerySet(models.query.QuerySet, HostRunMixin):
    pass


class HostRunManager(models.Manager, HostRunMixin):
    def get_query_set(self):
        return HostRunQuerySet(self.model, using=self._db)


class HostRun(models.Model):
    """
    M2M relationship between run and hosts -- Run on each Host
    """
    RUNNING = 'r'
    FINISHED = 'f'

    RUN_STATES = Enum([
        (RUNNING, 'running', 'Scan is active.'),
        (FINISHED, 'finished', 'Scan has finished.'),
    ])
    dt_finished = models.DateTimeField(blank=True, null=True)
    host = models.ForeignKey(Host)
    run = models.ForeignKey(Run)
    state = models.CharField(max_length=1, choices=RUN_STATES.get_mapping(), default=RUNNING, db_index=True)
    risk = models.CharField(max_length=16, blank=True, null=True, db_index=True)

    objects = HostRunManager()

    def __unicode__(self):
        return u"%s %s %s" % (self.get_state_display(), self.host, self.run)

    class Meta:
        ordering = ('-run__dt_submitted', )

    @property
    def running(self):
        return self.state == self.RUNNING

    @property
    def finished(self):
        return self.state == self.FINISHED

    def display_groups(self):
        return self.testgroupresult.filter(parent__isnull=True)

    def set_finished(self):
        self.state = self.FINISHED
        self.dt_finished = datetime.datetime.now()
        self.save(update_fields=["state"])

    def set_risk(self):
        risks = Risk.objects.for_hostrun(self)
        risk = "slight"
        if risks:
            risks_list = list(risks.values_list('level', flat=True).distinct())
            sorted_risks_list = sorted(risks_list, key=lambda x: Risk.RISK_LEVELS[x])
            risk = sorted_risks_list[-1]
        self.risk = risk
        self.save(update_fields=["risk"])

    def delete(self):
        if self.result:
            self.result.delete()
        super(HostRun, self).delete()


class Result(models.Model):
    """ report with results """
    dt_submitted = models.DateTimeField(blank=True, null=True)
    dt_finished = models.DateTimeField(blank=True, null=True)
    # html report
    filename = models.CharField(max_length=128, blank=True, null=True)
    hostrun = models.OneToOneField(HostRun)
    identity = models.CharField(max_length=255, blank=True, null=True)
    # hostname from results
    hostname = models.CharField(max_length=255, blank=True, null=True)

    test_count = models.SmallIntegerField(blank=True, null=True)
    failed_test_count = models.SmallIntegerField(blank=True, null=True)
    ni_test_count = models.SmallIntegerField(blank=True, null=True)
    na_test_count = models.SmallIntegerField(blank=True, null=True)

    def delete(self):
        result_dir = self.get_result_dir()
        super(Result, self).delete()
        rmtree(result_dir)

    def __unicode__(self):
        return u"%s (%s)" % (self.hostname, self.dt_finished)

    def get_file_path(self):
        return os.path.join(self.get_result_dir(), self.filename)

    def get_result_dir(self):
        run_dir = self.hostrun.run.get_run_dir()
        path = os.path.join(run_dir, str(self.id))
        if not os.path.isdir(path):
            os.makedirs(path, mode=0o0755)
        return path

    @property
    def results(self):
        return self.testresult_set.all()

    def status_text(self):
        result = []
        results_dict = TestResult.objects.for_result(self).count_states()
        for test in results_dict:
            test_print = "%s (%d)" % (
                TestResult.TEST_STATES.display(test['state']),
                test['count'],
            )
            result.append(test_print)
        return ', '.join(result)

    @property
    def groups(self):
        return self.testgroupresult_set.all()

    def root_groups(self):
        return self.testgroupresult_set.filter(group__parent__isnull=True)

    def states(self):
        """ return dict with states of tests in this run """

        result = {}

        for r in TestResult.objects.for_result(self).count_states():
            printable_state = TestResult.TEST_STATES.display(r['state'])
            print_text = "%s (%d)" % (printable_state, r['count'])
            result[TestResult.TEST_STATES[r['state']]] = print_text
            # e.g. {'fixed': 'Fixed (8)'}
        return result


class Address(models.Model):
    address = models.GenericIPAddressField()
    result = models.ForeignKey(Result)

    def __unicode__(self):
        return u"%s [%s]" % (self.address, self.result)


class TestMixin(object):
    def create_or_update(self, k):
        """
        return Test object for specified id_ref rule
        """
        try:
            test = self.get(id_ref=k['id_ref'])
        except ObjectDoesNotExist:
            test = Test(**k)
            test.save()
        else:
            self.filter(pk=test.pk).update(**k)
        return test


class TestQuerySet(models.query.QuerySet, TestMixin):
    pass


class TestManager(models.Manager, TestMixin):
    def get_query_set(self):
        return TestQuerySet(self.model, using=self._db)


class Test(models.Model):
    """ this model matches Rule of XCCDF """
    title = models.CharField(max_length=255, db_index=True)
    # rule's idref -- unique name of test within report
    id_ref = models.CharField(max_length=255)
    description = models.TextField()
    component = models.CharField(max_length=128, null=True, blank=True)
    # script for fixing
    fix = models.TextField(null=True, blank=True)
    fix_type = models.CharField(max_length=32, null=True, blank=True)
    fixtext = models.TextField(null=True, blank=True)
    group = models.ForeignKey(TestGroup, blank=True, null=True)
    objects = TestManager()

    def __unicode__(self):
        return u"%s %s" % (self.id_ref, self.title)


class TestResultMixin(object):
    def failed(self):
        return self.filter(state=TestResult.FAILURE)

    def ni(self):
        """ needs inspection """
        return self.filter(state=TestResult.NEEDS_INSPECTION)

    def na(self):
        """ needs action """
        return self.filter(state=TestResult.NEEDS_ACTION)

    def by_state(self, state):
        # TODO add validation if state is valid
        return self.filter(state=state)

    def by_states(self, states):
        # TODO add validation if state is valid
        return self.filter(state__in=states)

    def for_tgr(self, group):
        return self.filter(group=group)

    def search_in_title(self, search_string):
        return self.filter(test__title__icontains=search_string)

    def for_result(self, result):
        return self.filter(group__result=result)

    def count_states(self):
        """ return states with counts for current query """
        return self.values('state').annotate(count=Count('state'))


class TestResultQuerySet(models.query.QuerySet, TestResultMixin):
    pass


class TestResultManager(models.Manager, TestResultMixin):
    def get_query_set(self):
        return TestResultQuerySet(self.model, using=self._db)


class TestResult(models.Model):
    """ this model represents rule-result """
    # order, shortcut of the state
    PASSED = '06p'  # (p)assed
    WARNING = '03w'  # (w)arning
    FAILURE = '02f'  # (f)ailure
    NEEDS_INSPECTION = '03ni'  # (n)eeds (i)nspection
    NEEDS_ACTION = '03na'  # (n)eeds (a)ction
    ERROR = '01e'  # (e)rror
    NOTAPPLICABLE = '07n'  # (n)otapplicable
    NOTCHECKED = '08c'  # not(c)hecked
    INFO = '05i'  # (i)nfo
    FIXED = '04x'  # fi(x)ed

    TEST_STATES = Enum([
        # DB value, used in css/html, helper text, printable value
        (ERROR, 'error', 'Unexpected problem during analysis.', 'Error'),
        (FAILURE, 'fail', 'Test failed, some serious problems were found.', 'Failed'),
        (NEEDS_ACTION, 'needs_action', 'Issues found by this test should be resolved.', 'Needs Action'),
        (NEEDS_INSPECTION, 'needs_inspection', 'Issues found by this test should be inspected.', 'Needs Inspection'),
        (FIXED, 'fixed', 'Fixed.', 'Fixed'),
        (PASSED, 'pass', 'Test passed, no problems found.', 'Passed'),
        (INFO, 'informational', 'Test is only informative.', 'Informational'),
        (NOTAPPLICABLE, 'notapplicable', 'Test is not applicable.', 'Not Applicable'),
        #(WARNING, 'warning', 'Test passed but there are some issues.', 'warning'),
        (NOTCHECKED, 'notchecked', 'Test couldn\'t be run.', 'Not Checked'),
    ])
    state = models.CharField(max_length=3, choices=TEST_STATES.get_mapping(), db_index=True)
    date = models.DateTimeField()
    test = models.ForeignKey(Test)
    group = models.ForeignKey(TestGroupResult, related_name="testresult_set")
    root_group = models.ForeignKey(TestGroupResult, related_name="all_tests")
    # since groups are nested, there is no easy way to get to result from tr
    result = models.ForeignKey(Result)

    objects = TestResultManager()

    class Meta:
        ordering = ('state', )

    def __unicode__(self):
        return u"%s [%s] [%s]" % (self.get_state_display().upper(),
                                  self.test, self.group)

    def logs(self):
        return self.testlog_set.all()

    def risks(self):
        return self.risk_set.all()

    def set_state(self, state, save=True):
        self.state = self.TEST_STATES.get_key(state)
        if save:
            self.save()

    def get_state(self):
        return TestResult.TEST_STATES[self.state]

    def display_state(self):
        return TestResult.TEST_STATES.display(self.state)

    def should_display_solution(self):
        return self.get_state() not in ['pass', 'notapplicable']


class TestLogMixin(object):
    def bulk_create_logs(self, testlogs, result):
        """
        create hostruns for list of hosts specified as Query in variable hosts
        """
        testlog_list = []
        keys = ['date', 'level', 'message']

        for testlog in testlogs:
            testlog_dict = dict((key, testlog[key]) for key in keys if key in testlog)
            tl = TestLog(**testlog_dict)
            tl.result = result
            testlog_list.append(tl)
        TestLog.objects.bulk_create(testlog_list)


class TestLogQuerySet(models.query.QuerySet, TestLogMixin):
    pass


class TestLogManager(models.Manager, TestLogMixin):
    def get_query_set(self):
        return TestLogQuerySet(self.model, using=self._db)


class TestLog(models.Model):
    message = models.TextField()
    level = models.CharField(max_length=32)
    date = models.DateTimeField(null=True, blank=True)
    result = models.ForeignKey(TestResult)

    objects = TestLogManager()

    def __unicode__(self):
        return u"%s %s %s" % (self.level, self.date, self.message)


class RiskMixin(object):
    def for_hostrun(self, hostrun):
        risks_or = Q()
        risk_levels = Risk.RISK_LEVELS.keys()
        for r in risk_levels:
            risks_or |= Q(level=r)
        return self.filter(result__group__result__hostrun=hostrun).filter(risks_or)

    def bulk_create_logs(self, risks, result):
        """
        create risk objects in bulk: risks is a list of dicts
        """
        risks_list = []

        for risk in risks:
            tl = Risk(message=risk['message'], level=risk['level'].lower())
            tl.result = result
            risks_list.append(tl)
        Risk.objects.bulk_create(risks_list)


class RiskQuerySet(models.query.QuerySet, RiskMixin):
    pass


class RiskManager(models.Manager, RiskMixin):
    def get_query_set(self):
        return RiskQuerySet(self.model, using=self._db)


class Risk(models.Model):
    RISK_LEVELS = {
        "slight": 1,
        "medium": 2,
        "high": 3,
        "extreme": 4,
    }
    message = models.TextField()
    level = models.CharField(max_length=32)

    result = models.ForeignKey(TestResult)

    objects = RiskManager()

    def __unicode__(self):
        return u"%s %s" % (self.level, self.message)



# -*- coding: utf-8 -*-

"""
Import reports and figure out difference.
"""


class DifferenceItem(object):
    """
    runs may have several items which are different, each item is represented by this class

    E.g.

    |      Run 1      |     Run 2    |
    ----------------------------------
    | Test X          |              |
    |                 | Test Y       |
    | Test Z (failed) | Test Z (N/A) |


    Each line in this table is a DifferenceItem

    """
    def __init__(self):
        self.record = {}

    def add(self, **items):
        for item, value in items.iteritems():
            self.record[item] = value

    def display(self):
        return self.record


class TwoComparator(object):
    def __init__(self, left, right):
        self.left_result = left
        self.right_result = right
        self.diff = []

    def compare(self):
        left_tests = self.left_result.results.select_related().prefetch_related('testlog_set', 'risk_set', 'test')
        right_tests = self.right_result.results.select_related().prefetch_related('testlog_set', 'risk_set', 'test')

        left_mapping = {}
        for tr in left_tests:
            left_mapping[tr.test.id_ref] = tr
        right_mapping = {}
        for tr in right_tests:
            right_mapping[tr.test.id_ref] = tr

        for tr in left_tests:
            try:
                right_tr = right_mapping[tr.test.id_ref]
            except KeyError:
                di = DifferenceItem()
                di.add(left=tr)
                self.diff.append(di.display())
            else:
                if right_tr.state != tr.state:
                    di = DifferenceItem()
                    di.add(left=tr, right=right_tr)
                    self.diff.append(di.display())
        for tr in right_tests:
            try:
                left_mapping[tr.test.id_ref]
            except KeyError:
                di = DifferenceItem()
                di.add(right=tr)
                self.diff.append(di.display())
        return self.diff



from __future__ import unicode_literals
import unittest
import tempfile
import shutil
import os

from preupg.xccdf import XccdfHelper
from preupg.utils import FileHelper
from preupg import settings
from preupg.settings import ModuleValues

try:
    import base
except ImportError:
    import tests.base as base


class TestRiskCheck(base.TestCase):

    risks = ['SLIGHT', 'MEDIUM', 'HIGH', 'EXTREME']

    def _copy_xccdf_file(self, update_text=None, update_return_value=None):
        temp_dir = tempfile.mkdtemp()
        xccdf_file = os.path.join(os.getcwd(), 'tests', 'generated_results', 'inplace_risk_test.xml')
        temp_file = os.path.join(temp_dir, 'all-xccdf.xml')
        shutil.copyfile(xccdf_file, temp_file)
        content = FileHelper.get_file_content(temp_file, 'rb', decode_flag=False)
        new_text = b'preupg.risk.%s: Test %s Inplace risk' % (update_text, update_text)
        if update_text is not None:
            content = content.replace(b'INPLACE_TAG', new_text)
        else:
            content = content.replace(b'INPLACE_TAG', "")
        content = content.replace(b'RESULT_VALUE', update_return_value)
        FileHelper.write_to_file(temp_file, 'wb', content)
        return temp_file

    def _update_xccdf_file(self, return_value, risk):
        temp_file = self._copy_xccdf_file(update_return_value=return_value,
                                          update_text=risk)
        return_value = XccdfHelper.check_inplace_risk(os.path.join(os.path.dirname(temp_file), 'all-xccdf.xml'), 0)
        shutil.rmtree(os.path.dirname(temp_file))
        return return_value

    def test_check_inplace_risk_high(self):
        self.assertEqual(self._update_xccdf_file('needs_action', 'HIGH'), ModuleValues.NEEDS_ACTION)

    def test_check_inplace_risk_medium(self):
        self.assertEqual(self._update_xccdf_file('needs_inspection', 'MEDIUM'), ModuleValues.NEEDS_INSPECTION)

    def test_check_inplace_risk_slight(self):
        self.assertEqual(self._update_xccdf_file('needs_inspection', 'SLIGHT'), ModuleValues.NEEDS_INSPECTION)

    def test_check_inplace_risk_extreme(self):
        self.assertEqual(self._update_xccdf_file('fail', 'EXTREME'), ModuleValues.FAIL)

    def test_fail_return_value(self):
        self.assertEqual(self._update_xccdf_file('fail', None), ModuleValues.FAIL)

    def test_pass_return_values(self):
        expected_value = 'pass'
        self.assertEqual(self._update_xccdf_file(expected_value, None), ModuleValues.PASS)
        for risk in self.risks:
            self.assertEqual(self._update_xccdf_file(expected_value, risk), ModuleValues.PASS)

    def test_fixed_return_values(self):
        expected_value = 'fixed'
        self.assertEqual(self._update_xccdf_file(expected_value, None), ModuleValues.FIXED)
        for risk in self.risks:
            self.assertEqual(self._update_xccdf_file(expected_value, risk), ModuleValues.FIXED)

    def test_informational_return_values(self):
        expected_value = 'informational'
        self.assertEqual(self._update_xccdf_file(expected_value, None), ModuleValues.INFORMATIONAL)
        for risk in self.risks:
            self.assertEqual(self._update_xccdf_file(expected_value, risk), ModuleValues.INFORMATIONAL)

    def test_not_applicable_return_values(self):
        expected_value = 'not_applicable'
        self.assertEqual(self._update_xccdf_file(expected_value, None), ModuleValues.NOT_ALL)
        for risk in self.risks:
            self.assertEqual(self._update_xccdf_file(expected_value, risk), ModuleValues.NOT_ALL)

    def test_error_return_values(self):
        expected_value = 'error'
        self.assertEqual(self._update_xccdf_file(expected_value, None), ModuleValues.ERROR)
        for risk in self.risks:
            self.assertEqual(self._update_xccdf_file(expected_value, risk), ModuleValues.ERROR)


class TestCombinedRiskCheck(base.TestCase):

    risks = ['SLIGHT', 'MEDIUM', 'HIGH', 'EXTREME']

    def _copy_xccdf_file(self, update_text=None, update_return_values=None):
        temp_dir = tempfile.mkdtemp()
        xccdf_file = os.path.join(os.getcwd(), 'tests', 'generated_results', 'inplace_combined_risk_test.xml')
        temp_file = os.path.join(temp_dir, 'all-xccdf.xml')
        shutil.copyfile(xccdf_file, temp_file)
        content = FileHelper.get_file_content(temp_file, 'rb', decode_flag=False)
        if update_text:
            if update_text[0] is not None:
                new_text = b'preupg.risk.%s: Test %s Inplace risk' % (update_text[0], update_text[0])
                if update_text is not None:
                    content = content.replace(b'INPLACE_TAG1', new_text)
                else:
                    content = content.replace(b'INPLACE_TAG1', "")
            if update_text[1] is not None:
                new_text = b'preupg.risk.%s: Test %s Inplace risk' % (update_text[1], update_text[1])
                if update_text is not None:
                    content = content.replace(b'INPLACE_TAG2', new_text)
                else:
                    content = content.replace(b'INPLACE_TAG2', "")
        content = content.replace(b'RESULT_VALUE1', update_return_values[0])
        content = content.replace(b'RESULT_VALUE2', update_return_values[1])
        FileHelper.write_to_file(temp_file, 'wb', content)
        return temp_file

    def _update_xccdf_file(self, return_value, risk):
        temp_file = self._copy_xccdf_file(update_return_values=return_value,
                                          update_text=risk)
        return_value = XccdfHelper.check_inplace_risk(os.path.join(os.path.dirname(temp_file), 'all-xccdf.xml'), 0)
        shutil.rmtree(os.path.dirname(temp_file))
        return return_value

    def test_error_pass(self):
        self.assertEqual(self._update_xccdf_file(['error', 'pass'], [None, None]), ModuleValues.ERROR)

    def test_error_failed(self):
        self.assertEqual(self._update_xccdf_file(['error', 'fail'], [None, None]), ModuleValues.ERROR)

    def test_error_informational(self):
        self.assertEqual(self._update_xccdf_file(['error', 'informational'], [None, None]), ModuleValues.ERROR)

    def test_needs_inspection_pass(self):
        self.assertEqual(self._update_xccdf_file(['needs_inspection', 'pass'], [None, None]), ModuleValues.NEEDS_INSPECTION)

    def test_informational_pass(self):
        self.assertEqual(self._update_xccdf_file(['informational', 'pass'], [None, None]), ModuleValues.INFORMATIONAL)

    def test_fixed_informational(self):
        self.assertEqual(self._update_xccdf_file(['fixed', 'informational'], [None, None]), ModuleValues.FIXED)

    def test_fixed_pass(self):
        self.assertEqual(self._update_xccdf_file(['fixed', 'pass'], [None, None]), ModuleValues.FIXED)

    def test_pass_pass(self):
        self.assertEqual(self._update_xccdf_file(['pass', 'pass'], [None, None]), ModuleValues.PASS)

    def test_needs_action_failed(self):
        self.assertEqual(self._update_xccdf_file(['needs_action', 'fail'], ['HIGH', 'EXTREME']), ModuleValues.FAIL)

    def test_fixed_failed(self):
        self.assertEqual(self._update_xccdf_file(['fixed', 'fail'], [None, 'EXTREME']), ModuleValues.FAIL)

    def test_fixed_failed_none(self):
        self.assertEqual(self._update_xccdf_file(['fixed', 'fail'], [None, None]), ModuleValues.FAIL)

    def test_fixed_information_risk(self):
        self.assertEqual(self._update_xccdf_file(['fixed', 'informational'], [None, 'HIGH']), ModuleValues.FIXED)

    def test_not_applicable_pass(self):
        self.assertEqual(self._update_xccdf_file(['not_applicable', 'pass'], [None, None]), ModuleValues.NOT_ALL)


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestRiskCheck))
    suite.addTest(loader.loadTestsFromTestCase(TestCombinedRiskCheck))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



from __future__ import print_function, unicode_literals
import re
import os
import shutil

from preupg.utils import FileHelper
from preupg.xccdf import XccdfHelper
from preupg import settings
from preupg.logger import logger_report, log_message
try:
    from xml.etree import ElementTree
except ImportError:
    from elementtree import ElementTree
from preupg.xmlgen import xml_tags


def get_node(tree, tag, name_space='', prefix=''):
    return tree.find(prefix + name_space + tag)


def remove_node(tree, tag):
    return tree.remove(tag)


class ReportHelper(object):

    @staticmethod
    def get_needs_inspection():
        return settings.needs_inspection

    @staticmethod
    def get_needs_action():
        return settings.needs_action

    @staticmethod
    def upd_inspection(rule):
        """
        Function updates result to needs_inspection in case
        of SLIGHT or MEDIUM risk
        """
        return rule.get("idref"), ReportHelper.get_needs_inspection()

    @staticmethod
    def upd_action(rule):
        """Function updates result to needs_action in caseof HIGH"""
        return rule.get("idref"), ReportHelper.get_needs_action()

    @staticmethod
    def upd_extreme(rule):
        """Function does no update result for extreme risk"""
        return None, "fail"


class ReportParser(object):

    """Class manipulates with XML files created by oscap"""

    def __init__(self, report_path):
        self.path = report_path
        self.element_prefix = "{http://checklists.nist.gov/xccdf/1.2}"
        # ElementTree.fromstring can't parse safely unicode string
        content = FileHelper.get_file_content(report_path, 'rb', False, False)
        self.target_tree = ElementTree.fromstring(content)
        self.profile = "Profile"
        self.changed_results = {}
        self.output_data = []

    def filter_children(self, tree, tag):
        return self.get_nodes(tree, tag, prefix='./')

    def has_children(self, tree, tag):
        return len(self.get_nodes(tree, tag, prefix='./')) > 0

    def filter_grandchildren(self, tree, parent_tag, tag):
        return tree.findall('./%s%s/%s%s' % (self.element_prefix,
                                             parent_tag,
                                             self.element_prefix, tag))

    def get_child(self, tree, tag):
        return get_node(tree, tag, self.element_prefix, prefix='./')

    def get_nodes_attrib(self, tree, tag, attrib):
        try:
            return self.get_child(tree, tag).attrib[attrib]
        except (KeyError, AttributeError):
            return ''

    def get_nodes(self, tree, tag, prefix=''):
        return tree.findall(prefix + self.element_prefix + tag)

    def get_nodes_text(self, tree, tag):
        try:
            text = self.get_child(tree, tag).text
        except AttributeError:
            pass
        else:
            if text:
                return text.strip()
        return ''

    def reload_xml(self, path):
        """Function updates self.target_tree with the new path"""
        self.path = path
        # ElementTree.fromstring can't parse safely unicode string
        content = FileHelper.get_file_content(self.path, 'rb', False, False)
        if not content:
            return None

        # remove the BEL characters from the loaded string, because
        # this character causes crash of the parsing function of ElementTree
        # when it occurs
        self.target_tree = ElementTree.fromstring(content.replace("\a", ""))

    def get_select_rules(self):
        selected = self.filter_grandchildren(self.target_tree, self.profile, "select")
        return selected

    def get_allowed_selected_rules(self):
        selected = []
        for sel in self.get_select_rules():
            if sel.get('selected') == 'true':
                selected.append(sel)
        return selected

    def get_number_checks(self):
        """Function returns a number of checkswho are really selected"""
        return len(self.get_allowed_selected_rules())

    def _get_all_rules(self):
        return self.get_nodes(self.target_tree, "Rule", prefix=".//")

    def get_name_of_checks(self):
        """Function returns a names of rules"""
        list_names = {}
        rule_nodes = self._get_all_rules()
        for select in self.get_allowed_selected_rules():
            id_ref = select.get('idref', '')
            rule = [x for x in rule_nodes if x.get('id', '') == id_ref]
            list_names[id_ref] = self.get_nodes_text(rule[0], "title")
        return list_names

    def get_all_result_rules(self):
        """Function returns all rul-result in TestResult xml tag"""
        return self.filter_grandchildren(self.target_tree, "TestResult", "rule-result")

    def get_all_results(self):
        """Function return all results"""
        results = []
        for rule in self.get_all_result_rules():
            results.extend(self.get_nodes(rule, "result", prefix='./'))
        return results

    def write_xml(self):
        """Function writes XML document to file"""
        self.target_tree.set('xmlns:xhtml', 'http://www.w3.org/1999/xhtml/')
        # we really must set encoding here! and suppress it in write_to_file
        data = ElementTree.tostring(self.target_tree, "utf-8")
        FileHelper.write_to_file(self.path, 'wb', data, False)
        self.target_tree = ElementTree.parse(self.path).getroot()

    def modify_result_path(self, result_dir, scenario, mode):
        """Function modifies result path in XML file"""
        for values in self.get_nodes(self.target_tree, "Value", prefix='.//'):
            if '_current_dir' not in values.get('id'):
                continue
            for value in self.get_nodes(values, "value"):
                logger_report.debug("Replace '%s' with '%s'", value.text, os.path.join(result_dir, scenario))
                value.text = value.text.replace("SCENARIO", os.path.join(result_dir, scenario))

        self.write_xml()

    def update_inplace_risk(self, scanning_progress, rule, res):
        """Function updates inplace risk"""
        inplace_risk = XccdfHelper.get_check_import_inplace_risk(rule)
        if inplace_risk:
            logger_report.debug("Update_inplace_risk '%s'", inplace_risk)
            return_value = XccdfHelper.get_and_print_inplace_risk(0, inplace_risk)
            logger_report.debug("Get and print inplace risk return code '%s'", return_value)
            if int(return_value)/2 == 1:
                res.text = ReportHelper.get_needs_inspection()
            elif int(return_value)/2 == 2:
                res.text = ReportHelper.get_needs_action()
            for index, row in enumerate(scanning_progress.output_data):
                if self.get_nodes_text(rule, "title") in row:
                    scanning_progress.output_data[index] = "{0}:{1}".format(
                        self.get_nodes_text(rule, "title"),
                        res.text)

    def replace_inplace_risk(self, scanning_results=None):
        """
        This function has aim to replace FAILED to
        NEEDS_INSPECTION in case that risks are SLIGHT or MEDIUM
        """
        #Filter all rule-result in TestResult
        changed_fields = []
        self.remove_empty_check_import()
        inplace_dict = {
            0: ReportHelper.upd_inspection,
            1: ReportHelper.upd_action,
            2: ReportHelper.upd_extreme,
        }
        for rule in self.get_all_result_rules():
            result = [x for x in self.get_nodes(rule, "result") if x.text == "fail"]
            # Get all affected rules and taken their names
            for res in result:
                inplace_risk = XccdfHelper.get_check_import_inplace_risk(rule)
                logger_report.debug(inplace_risk)
                # In case that report has state fail and
                # no log_risk than it should be needs_inspection
                if not inplace_risk:
                    changed = rule.get("idref")
                    res.text = "fail"
                    log_message('The %s module exits as fail but without a risk.' % rule.get("idref"))
                else:
                    inplace_num = XccdfHelper.get_and_print_inplace_risk(0, inplace_risk)
                    logger_report.debug("Call function '%s'", inplace_dict[inplace_num])
                    changed, res.text = inplace_dict[inplace_num](rule)
                    logger_report.debug("Replace text '%s:%s'", changed, res.text)
                if changed is not None:
                    changed_fields.append(changed+":"+res.text)

        if scanning_results:
            scanning_results.update_data(changed_fields)

        self.write_xml()

    def remove_empty_check_import(self):
        """Remove stdout or stderr check-import tags whose text is either empty
        or contains nothing but whitespace characters.
        """
        has_std_name = lambda x: x.get("import-name") in ["stdout", "stderr"]
        is_empty = lambda x: x.text is None or x.text.strip() == ""
        for rule in self.get_all_result_rules():
            for check in self.get_nodes(rule, "check"):
                for node in self.get_nodes(check, "check-import"):
                    if has_std_name(node) and is_empty(node):
                        remove_node(check, node)

    def remove_debug_info(self):
        """Function removes debug information from report"""
        re_expr = r'^preupg.log.DEBUG.*'
        for rule in self.get_all_result_rules():
            for check_import in self.filter_grandchildren(rule,
                                                          "check",
                                                          "check-import"):
                if check_import.text is not None:
                    new_check = []
                    for check in check_import.text.split('\n'):
                        matched = re.match(re_expr, check)
                        if not matched:
                            new_check.append(check)
                    check_import.text = '\n'.join(new_check)
        self.write_xml()

    def strip_whitespaces(self):
        """Strip specific whitespace characters from the start and end of
        stderr/stdout of modules.
        """
        check_imports = self.target_tree.getiterator(self.element_prefix +
                                                     "check-import")
        for check_import in check_imports:
            if check_import.get("import-name") not in ["stdout", "stderr"]:
                continue
            txt = check_import.text
            if txt and txt.startswith('\n') and \
                    txt.endswith('\n\n          \n'):
                check_import.text = txt[1:-13]

    @staticmethod
    def write_xccdf_version(file_name, direction=False):
        """
        Function updates XCCDF version because
        of separate HTML generation and our own XSL stylesheet
        """
        namespace_1 = 'http://checklists.nist.gov/xccdf/1.1'
        namespace_2 = 'http://checklists.nist.gov/xccdf/1.2'
        content = FileHelper.get_file_content(file_name, "rb")
        if direction:
            content = re.sub(namespace_2, namespace_1, content)
        else:
            content = re.sub(namespace_1, namespace_2, content)
        FileHelper.write_to_file(file_name, 'wb', content)

    def update_check_description(self):
        logger_report.debug("Update check description")
        for rule in self._get_all_rules():
            for description in self.filter_children(rule, 'description'):
                lines = description.text.split('\n')
                details = 'Details:'
                expected_results = 'Expected results:'
                attr = ' xml:lang="en"'
                tag_exp_results = expected_results[:-1].lower().replace(' ', '-')
                tag_details = details[:-1].lower()
                found = 0
                for index, line in enumerate(lines):
                    if line.strip().startswith(details):
                        found = 1
                        lines[index] = line.replace(details, '<ns0:' + tag_details + attr + '>')
                        continue
                    if line.strip().startswith(expected_results):
                        found = 1
                        lines[index] = line.replace(expected_results,
                                                    '</ns0:' + tag_details + '>\n<ns0:' + tag_exp_results + attr + '>')
                        continue
                if found == 1:
                    lines.append('</ns0:' + tag_exp_results + '>')
                    description.text = '\n'.join(lines)
        self.write_xml()

    def select_rules(self, list_rules):
        """
        Function marks choice a specific rules based on the content generation

        :return:
        """
        for select in self.get_select_rules():
            idref = select.get('idref', None)
            logger_report.debug(select)
            logger_report.debug(idref)
            if idref in list_rules:
                select.set('selected', 'true')
            else:
                select.set('selected', 'false')
        self.write_xml()

    def check_rules(self, list_rules):
        """
        Function checks if rules exists
        :param list_rules:
        :return: List of rules which does not exist
        """
        unknown_rules = []
        for select in list_rules:
            found = [i for i in self.get_select_rules() if select in i.get('idref')]
            if not found:
                unknown_rules.append(select)
        return unknown_rules

    def list_rules(self):
        list_rules = []
        for select in self.get_select_rules():
            idref = select.get('idref', None)
            if idref:
                list_rules.append(idref)
        return list_rules

    def get_path(self):
        """Function return path to report"""
        return self.path

    def add_global_tags(self, result_dir, scenario, mode, devel_mode, dist_native):

        for child in self.get_nodes(self.target_tree, self.profile):
            last_child = child
            for key, val in iter(xml_tags.GLOBAL_DIC_VALUES.items()):
                if key == "tmp_preupgrade":
                    val = result_dir
                elif key == "migrate" or key == "upgrade":
                    if not mode or 'migrate' in mode or 'upgrade' in mode:
                        val = "1"
                    else:
                        val = "0"
                elif key == "report_dir":
                    val = os.path.join(result_dir, scenario)
                elif key == "devel_mode":
                    val = str(devel_mode)
                elif key == "dist_native":
                    if dist_native is None:
                        val = "sign"
                    else:
                        val = dist_native
                logger_report.debug("'%s:%s'", key, val)
                new_child = ElementTree.Element(self.element_prefix + 'Value',
                                                {'id': xml_tags.TAG_VALUE + key,
                                                 'type': 'string'
                                                 })
                sub_child = ElementTree.SubElement(new_child, self.element_prefix + 'value')
                sub_child.text = val
                self.target_tree.insert(self.target_tree._children.index(last_child) + 1,
                                        new_child)
        self.write_xml()



# -*- coding: utf-8 -*-
"""
python API for content writers

USAGE
*****

Best way is to import all functions from this module:

from script_api import *

These functions are available:

* logging functions -- log_*
 * log message to stdout
* logging risk functions -- log_*_risk
 * log risk level -- so administrator know how risky is to inplace upgrade
* get_dest_dir -- get dir for storing configuration files
* exit_* -- terminate execution with appropriate exit code
"""

from __future__ import unicode_literals, print_function
import os
import sys
import re
import shutil
import errno
try:
    import configparser
except ImportError:
    import ConfigParser as configparser

from preupg import settings
from preupg.utils import FileHelper, ProcessHelper

__all__ = (
    'log_debug',
    'log_info',
    'log_warning',
    'log_error',

    'log_extreme_risk',
    'log_high_risk',
    'log_medium_risk',
    'log_slight_risk',

    'get_dest_dir',

    'exit_error',
    'exit_fail',
    'exit_fixed',
    'exit_informational',
    'exit_not_applicable',
    'exit_informational',
    'exit_pass',
    'check_rpm_to',
    'check_applies_to',
    'solution_file',
    'switch_to_content',
    'service_is_enabled',
    'is_dist_native',
    'get_dist_native_list',
    'is_pkg_installed',
    'add_pkg_to_kickstart',
    'deploy_hook',

    'PREUPGRADE_CACHE',
    'VALUE_RPM_QA',
    'VALUE_ALLCHANGED',
    'VALUE_CONFIGCHANGED',
    'VALUE_PASSWD',
    'VALUE_CHKCONFIG',
    'VALUE_GROUP',
    'VALUE_RPMTRACKEDFILES',
    'VALUE_ALLMYFILES',
    'VALUE_EXECUTABLES',
    'VALUE_RPM_RHSIGNED',
    'VALUE_TMP_PREUPGRADE',
    'MODULE_PATH',
    'COMMON_DIR',
    'SOLUTION_FILE',
    'POSTUPGRADE_DIR',
    'KICKSTART_DIR',
    'KICKSTART_README',
    'KICKSTART_SCRIPTS',
    'KICKSTART_POSTUPGRADE',
    'MIGRATE',
    'UPGRADE',
    'HOME_DIRECTORY_FILE',
    'USER_CONFIG_FILE',
    'PREUPG_API_VERSION',
    'DEVEL_MODE',
    'DIST_NATIVE',
    'SPECIAL_PKG_LIST',
    'NOAUTO_POSTUPGRADE_D',
)

# These variables and functions will be available
# for Bash preupgrade-assistant modules.
#

CACHE = "/var/cache/preupgrade"

#
# Directory with logs gathered by preupgrade-assistant
#
PREUPGRADE_CACHE = os.path.join(CACHE, "common")

#
# Preupgrade-assistant configuration file
#
PREUPGRADE_CONFIG = settings.PREUPG_CONFIG_FILE

#
# Full path log file with to all installed packages
#
VALUE_RPM_QA = os.path.join(PREUPGRADE_CACHE, "rpm_qa.log")

#
# Full path log file with to all changed files
#
VALUE_ALLCHANGED = os.path.join(PREUPGRADE_CACHE, "rpm_Va.log")

#
# Full path to log file with all /etc changed configuration files
#
VALUE_CONFIGCHANGED = os.path.join(PREUPGRADE_CACHE, "rpm_etc_Va.log")

#
# Full path to log file with all users gathered by getent
#
VALUE_PASSWD = os.path.join(PREUPGRADE_CACHE, "passwd.log")

#
# Full path to log file with all services enabled/disabled on system.
#
VALUE_CHKCONFIG = os.path.join(PREUPGRADE_CACHE, "chkconfig.log")

#
# Full path to log file with all groups gathered by getent
#
VALUE_GROUP = os.path.join(PREUPGRADE_CACHE, "group.log")

#
# Full path to log file with all installed files
#
VALUE_RPMTRACKEDFILES = os.path.join(PREUPGRADE_CACHE, "rpmtrackedfiles.log")

#
# Full path to log file with all installed files
#
VALUE_ALLMYFILES = os.path.join(PREUPGRADE_CACHE, "allmyfiles.log")

#
# Full path to log file with all executable files
#
VALUE_EXECUTABLES = os.path.join(PREUPGRADE_CACHE, "executable.log")

#
# Full path to log file with all Red Hat signed packages
#
VALUE_RPM_RHSIGNED = os.path.join(PREUPGRADE_CACHE, "rpm_rhsigned.log")

#
# Variable which referes to temporary directory directory provided by module
#
VALUE_TMP_PREUPGRADE = os.environ['XCCDF_VALUE_TMP_PREUPGRADE']

#
# Directory with configuration files that can be applied safely.
#
# Configuration files in this directory will be automatically applied on the
# upgraded system. Files has to be stored in this directory using whole path
# referring to the place where they should be copied. E.g.:
#   $CLEANCONF_DIR/etc/group -> /etc/group
#
CLEANCONF_DIR = os.path.join(VALUE_TMP_PREUPGRADE, "cleanconf")

#
# Directory with configuration files that need to be overviewed manually.
#
# Configuration files in this directory cannot be applied on the upgraded
# system safely and need to be handled or overviewed manually. Usually are not
# copied automatically on the upgraded system unless there is a post-upgrade
# script that handle issue related with a configuration file at least
# partially.
#
DIRTYCONF_DIR = os.path.join(VALUE_TMP_PREUPGRADE, "dirtyconf")

#
# Variable which referes to current directory directory provided by module
#
VALUE_CURRENT_DIRECTORY = os.environ['XCCDF_VALUE_CURRENT_DIRECTORY']

#
# Variable which referes to solution file provided by module
#
SOLUTION_FILE = os.path.join(VALUE_CURRENT_DIRECTORY, settings.solution_txt)

#
# Variable which referes to current upgrade path directory
#
VALUE_REPORT_DIR = os.environ['XCCDF_VALUE_REPORT_DIR']

#
# Name of module being currently executed
#
try:
    MODULE_PATH = os.environ['XCCDF_VALUE_MODULE_PATH']
except KeyError:
    MODULE_PATH = VALUE_CURRENT_DIRECTORY.replace(VALUE_REPORT_DIR, '')
    MODULE_PATH = MODULE_PATH.replace('/', '_')


#
# MIGRATE means if preupg binary was used with `--mode migrate` parameter
# UPGRADE means if preupg binary was used with `--mode upgrade` parameter
# These modes are used if `--mode` is not used
#
try:
    MIGRATE = os.environ['XCCDF_VALUE_MIGRATE']
    UPGRADE = os.environ['XCCDF_VALUE_UPGRADE']
except KeyError:
    MIGRATE = 1
    UPGRADE = 1

#
# Variable which indicates DEVEL mode.
#
try:
    DEVEL_MODE = os.environ['XCCDF_VALUE_DEVEL_MODE']
except KeyError:
    DEVEL_MODE = 0

#
# Override mode for is_dist_native() and similar
#
# Affects which packages are considered native:
#
# If set to 'sign' (default), GPG signature is consulted.  If 'all',
# all packages are native.  If set to path to a file, packages listed
# there are native.
#
try:
    DIST_NATIVE = os.environ['XCCDF_VALUE_DIST_NATIVE']
except KeyError:
    DIST_NATIVE = 'sign'

#
# preupgrade-scripts directory used by redhat-upgrade-tool
#
# Executable scripts inside the directrory (ans subdirectories) are processed
# by redhat-upgrade-tool during the pre-upgrade phase, after the upgrade RPM
# transaction is calculated and before the reboot is processed.
#
PREUPGRADE_SCRIPT_DIR = os.path.join(VALUE_TMP_PREUPGRADE, "preupgrade-scripts")

#
# postupgrade directory used by in-place upgrades.
#
# Scripts mentioned there are executed automatically by redhat-upgrade-tool
#
POSTUPGRADE_DIR = os.path.join(VALUE_TMP_PREUPGRADE, "postupgrade.d")

#
# postmigrate directory used after migration
#
# Executable scripts in the directory are processed during the %post phase
# when migration to the new system is done using the generated kickstart file.
#
POSTMIGRATE_DIR = os.path.join(VALUE_TMP_PREUPGRADE, "postmigrate.d")

#
# Directory which is used for kickstart generation
#
KICKSTART_DIR = os.path.join(VALUE_TMP_PREUPGRADE, "kickstart")

#
# README file which contains description about all files in kickstart directory
#
KICKSTART_README = os.path.join(KICKSTART_DIR, "README")

#
# Directory with scripts which can be executed after installation by administrator
#
KICKSTART_SCRIPTS = os.path.join(KICKSTART_DIR, "scripts")

#
# The same as $KICKSTART_SCRIPTS
#
KICKSTART_POSTUPGRADE = KICKSTART_SCRIPTS

#
# Variable which refers to static data used by preupgrade-assistant and modules
#
COMMON_DIR = os.path.join(os.environ['XCCDF_VALUE_REPORT_DIR'], "common")

#
# Variable which contains file with packages add to the kickstart anyway
#
SPECIAL_PKG_LIST = os.path.join(KICKSTART_DIR, 'special_pkg_list')

#
# Postupgrade directory which is not executed automatically after an upgrade or migration
#
NOAUTO_POSTUPGRADE_D = os.path.join(VALUE_TMP_PREUPGRADE, 'noauto_postupgrade.d')

#
# variables set by PA config file #
#
HOME_DIRECTORY_FILE = ""
USER_CONFIG_FILE = 0

#
# Version of this API
#
PREUPG_API_VERSION = 1

################
# RISK LOGGING #
################

def _log_risk(severity, message):
    """
    log risk level to stderr
    """
    print("preupg.risk.%s: %s\n" % (severity, message.encode(settings.defenc)), end="", file=sys.stderr)


def log_extreme_risk(message):
    """
    log_extreme_risk(message)

    Inplace upgrade is impossible.
    """
    _log_risk("EXTREME", message)


def log_high_risk(message):
    """
    log_high_risk(message)

    Administrator has to inspect and correct upgraded system so
    inplace upgrade can be used.
    """
    _log_risk("HIGH", message)


def log_medium_risk(message):
    """
    log_medium_risk(message)

    inplace upgrade is possible; system after upgrade may be unstable
    """
    _log_risk("MEDIUM", message)


def log_slight_risk(message):
    """
    log_slight_risk(message)

    no issues found; although there are some unexplored areas
    """
    _log_risk("SLIGHT", message)


##################
# STDOUT LOGGING #
##################

def _log(severity, message):
    """
    general logging function

    :param severity: set it to one of INFO|ERROR|WARNING
    :param message:message to be logged
    :return:
    """
    print("preupg.log.%s: %s\n" % (severity, message.encode(settings.defenc)), end="", file=sys.stderr)


def log_error(message):
    """
    log_error(message) -> None

    log message to stdout with severity error

    use this severity if your script found something severe
    which may cause malfunction on new system
    """
    _log('ERROR', message)


def log_warning(message):
    """
    log_warning(message) -> None

    log message to stdout with severity warning

    important finding, administrator of system should be aware of this
    """
    _log('WARNING', message)


def log_info(message):
    """
    log_info(message) -> None

    log message to stdout with severity info

    informational message
    """
    _log('INFO', message)


def log_debug(message):
    """
    log_debug(message) -> None

    log message to stdout with severity debug

    verbose information, may help with script debugging
    """
    _log('DEBUG', message)


#########
# UTILS #
#########

def get_dest_dir():
    """
    get_dest_dir()

    return absolute path to directory, where you should store files
    """
    return os.environ['XCCDF_VALUE_TMP_PREUPGRADE']


def shorten_envs():
    """make all the oscap's environemt variables shorter"""
    envs = os.environ
    prefixes = ('XCCDF_VALUE_', 'XCCDF_RESULT_')
    for env_key, env_value in envs.items():
        for prefix in prefixes:
            if env_key.startswith(prefix):
                os.environ[env_key.replace(prefix, '')] = env_value


# These are shortcut functions for:
#   sys.exit(int(os.environ['FAIL']))

def exit_fail():
    """
    The test failed.
    Moving to new release with this configuration will result in malfunction.
    """
    sys.exit(int(os.environ['XCCDF_RESULT_FAIL']))


def exit_error():
    """
    An error occurred and test could not complete.

    (script failed while doing its job)
    """
    sys.exit(int(os.environ['XCCDF_RESULT_ERROR']))


def exit_pass():
    """Test passed."""
    sys.exit(int(os.environ['XCCDF_RESULT_PASS']))


def exit_not_applicable():
    """Rule did not apply to test target. (e.g. package is not installed)"""
    sys.exit(int(os.environ['XCCDF_RESULT_NOT_APPLICABLE']))


def exit_fixed():
    """Rule failed, but was later fixed."""
    sys.exit(int(os.environ['XCCDF_RESULT_FIXED']))


def exit_informational():
    """Rule failed, but was later fixed."""
    sys.exit(int(os.environ['XCCDF_RESULT_INFORMATIONAL']))


def switch_to_content():
    """Function for switch to the content directory"""
    os.chdir(os.environ['CURRENT_DIRECTORY'])


def is_pkg_installed(pkg_name):
    """
    Function checks if package is installed.

    :param pkg_name: Parameter is a package name which will be checked.
    :return: 0 - package is installed
             1 - package is NOT installed
    """
    lines = FileHelper.get_file_content(VALUE_RPM_QA, "rb", True)
    found = [x for x in lines if x.split()[0] == pkg_name]
    if found:
        return True
    else:
        return False


def check_applies_to(check_applies=""):
    """
    Function checks is package is installed and signed by Red Hat

    :param check_applies: Parameter list of packages which will be checked. Module requires them.
    :return: 0 - package is installed and signed by Red Hat
             exit_not_applicable - module will not be executed
    """
    not_applicable = 0
    if check_applies != "":
        rpms = check_applies.split(',')
        for rpm in rpms:
            if not (is_pkg_installed(rpm) and is_dist_native(rpm)):
                log_info("Package %s is not installed or it is not signed by Red Hat." % rpm)
                not_applicable = 1
    if not_applicable:
        exit_not_applicable()
    return not_applicable


def check_rpm_to(check_rpm="", check_bin=""):
    """
    Function checks if relevant package is installed and if relevant binary exists on the system.

    Function is needed from module point of view.

    :param check_rpm: list of RPMs separated by comma
    :param check_bin: list of binaries separated by comma
    :return:
    """
    not_applicable = 0

    if check_rpm != "":
        rpms = check_rpm.split(',')
        lines = FileHelper.get_file_content(VALUE_RPM_QA, "rb", True)
        for rpm in rpms:
            lst = [x for x in lines if rpm == x.split('\t')[0]]
            if not lst:
                log_high_risk("Package %s is not installed." % rpm)
                not_applicable = 1

    if check_bin != "":
        binaries = check_bin.split(',')
        for binary in binaries:
            cmd = "which %s" % binary
            if ProcessHelper.run_subprocess(cmd, print_output=False, shell=True) != 0:
                log_high_risk("Binary %s is not installed." % binary)
                not_applicable = 1

    if not_applicable:
        log_high_risk("Please, install all required packages (and binaries)"
                      " and run preupg again to process check properly.")
        exit_fail()
    return not_applicable


def solution_file(message):
    """
    Function appends a message to solution file.

    solution file will be created in module directory

    :param message: Message - string of list of strings
    :return:
    """
    if os.path.exists(SOLUTION_FILE):
        mod = "a+b"
    else:
        mod = "wb"
    FileHelper.write_to_file(SOLUTION_FILE, mod, message)


def service_is_enabled(service_name):
    """Returns true if given service is enabled on any runlevel"""
    return_value = False
    lines = FileHelper.get_file_content(VALUE_CHKCONFIG, "rb", True)
    for line in lines:
        if re.match('^%s.*:on' % service_name, line):
            return_value = True
            break
    return return_value


def config_file_changed(config_file_name):
    """
    Searches cached data in VALUE_CONFIGCHANGED

    returns:
    True if given config file has been changed
    False if given config file hasn't been changed
    """
    config_changed = False
    try:
        lines = FileHelper.get_file_content(VALUE_CONFIGCHANGED, "rb", True)
        for line in lines:
            if line.find(config_file_name) != -1:
                config_changed = True
                break
    except:
        pass
    return config_changed


def backup_config_file(config_file_name):
    """
    backup the config file

    :param config_file_name:
    :return:
            true if cp succeeds,
            if config file doesn't exist
            2 if config file was not changed and thus is not necessary to back-up
    """
    try:
        # report error if file doesn't exist
        if not os.path.isfile(config_file_name):
            return 1

        # don't do anything if config file was not changed
        if not config_file_changed(config_file_name):
            return 2

        # stripping / from beginning is necessary to concat paths properly
        os.mkdir(os.path.join(VALUE_TMP_PREUPGRADE, os.path.dirname(config_file_name.strip("/"))))
    except OSError:
        # path probably exists, it's ok
        pass
    shutil.copyfile(config_file_name, os.path.join(VALUE_TMP_PREUPGRADE, config_file_name.strip("/")))


def is_dist_native(pkg):
    """
    return 1 if package is not installed and print warning log.
    is_dist_native function return only True or False
    Case DEVEL_MODE is turn off then return True if package is signed or False if not.
    Case DEVEL_MODE is turn on:
    DIST_NATIVE = sign: return True if is RH_SIGNED else return False
    DIST_NATIVE = all: always return True
    DIST_NATIVE = path_to_file: return True if package is in file else return False
    """

    rpm_qa = FileHelper.get_file_content(VALUE_RPM_QA, "rb", True)
    found = [x for x in rpm_qa if x.split()[0] == pkg]
    if not found:
        log_warning("Package %s is not installed on Red Hat Enterprise Linux system.")
        return False

    rpm_signed = FileHelper.get_file_content(VALUE_RPM_RHSIGNED, "rb", True)
    found = [x for x in rpm_signed if x.split()[0] == pkg]

    if int(DEVEL_MODE) == 0:
        if found:
            return True
        else:
            return False
    else:
        if DIST_NATIVE == "all":
            return True
        if DIST_NATIVE == "sign":
            if found:
                return True
            else:
                return False
        if os.path.exists(DIST_NATIVE):
            list_native = map(
                unicode.strip,
                FileHelper.get_file_content(DIST_NATIVE, "r", method=True)
            )
            if pkg in list_native:
                return True
        return False


def get_dist_native_list():
    """
    return list of all dist native packages according to is_dist_native()
    """

    native_pkgs = []
    tmp = FileHelper.get_file_content(VALUE_RPM_QA, "rb", True)
    pkgs = [i.split("\t")[0] for i in tmp]
    for pkg in pkgs:
        if is_dist_native(pkg) is True:
            native_pkgs.append(pkg)
    return native_pkgs


def load_pa_configuration():
    """ this is main function for parsing """
    global HOME_DIRECTORY_FILE
    global USER_CONFIG_FILE
    global RH_SIGNED_PKGS

    if not os.path.exists(PREUPGRADE_CONFIG):
        log_error("Configuration file %s is missing or is not readable!"
                  % PREUPGRADE_CONFIG)
        exit_error()

    config = configparser.RawConfigParser()
    config.read(PREUPGRADE_CONFIG)
    section = 'preupgrade-assistant'
    home_option = 'home_directory_file'
    user_file = 'user_config_file'
    if config.has_section(section):
        if config.has_option(section, home_option):
            HOME_DIRECTORY_FILE = config.get(section, home_option)
        if config.has_option(section, user_file):
            USER_CONFIG_FILE = config.get(section, user_file)


def print_home_dirs(user_name=""):
    """
    print items from [home-dirs] which are relevant for given user

    when username is not given or config file for user is not enabled,
    items from main configuration file is printed
    returns 0 on SUCCESS, otherwise 1 and logs warning
    shouldn't be used before load_config_parser
    """
    if not os.path.exists(PREUPGRADE_CONFIG):
        log_error("Configuration file %s is missing or is not readable!"
                  % PREUPGRADE_CONFIG)
        exit_error()

    config = configparser.RawConfigParser()
    home_option = 'home-dirs'
    try:
        if USER_CONFIG_FILE == 'enabled' and user_name == "":
            config.read(PREUPGRADE_CONFIG)
            return config.options(home_option)
        user_home_dir = os.path.join('/home', user_name, HOME_DIRECTORY_FILE)
        if not os.path.exists(user_home_dir):
            return 0
        config.read(user_home_dir)
        return config.options(home_option)
    except configparser.NoSectionError:
        pass
    except configparser.NoOptionError:
        pass


def add_pkg_to_kickstart(pkg_name):
    """ Function adds a package to special_pkg_list """
    empty = False
    if isinstance(pkg_name, list):
        # list of packages
        if len(pkg_name) == 0:
            empty = True
    else:
        # string - pkg_name delimited by whitespace
        if len(pkg_name.strip()) == 0:
            empty = True
        else:
            # make list from string
            pkg_name = pkg_name.strip().split()
    if empty is True:
        log_debug("Missing parameters! Any package will be added.")
        return 1
    for pkg in pkg_name:
        FileHelper.write_to_file(SPECIAL_PKG_LIST, "a+b", pkg.strip() + '\n')
    return 0


def deploy_hook(*args):
    """
    Function which deploys script to specific location.

    :param hook: hook, like postupgrade, preupgrade, etc.
    :param script_name: script name
    :return:
    """

    if MODULE_PATH == "":
        return 0
    if len(args) < 1:
        log_error("Hook name is not specified. (Possible values are postupgrade, preupgrade.)")
        exit_error()
    elif len(args) < 2:
        log_error("Script name is not specified. It is mandatory.")
        exit_error()
    deploy_name = args[0]
    script_name = args[1]
    if deploy_name == "postupgrade" or deploy_name == "preupgrade":
        if not os.path.exists(script_name):
            log_error("Script_name %s does not exist." % script_name)
            exit_error()
        hook_dir = "%s/hooks/%s/%s" % (VALUE_TMP_PREUPGRADE, MODULE_PATH, deploy_name)
        if not os.path.isdir(hook_dir):
            os.makedirs(hook_dir)
        else:
            log_error("The %s directory already exists" % hook_dir)
            exit_error()
        try:
            shutil.copyfile(script_name, os.path.join(hook_dir, "run_hook"))
            for arg in args[2:]:
                if arg.startswith('/'):
                    hook_arg = os.path.join(hook_dir, os.path.basename(arg))
                else:
                    hook_arg = os.path.join(hook_dir, arg)
                if os.path.isdir(hook_arg):
                    log_error("The %s directory already exists" % hook_arg)
                    exit_error()
                if os.path.isfile(hook_arg):
                    log_error("The %s file already exists" % hook_arg)
                    exit_error()
                try:
                    shutil.copytree(arg, hook_arg)
                except OSError as exc:
                    if exc.errno == errno.ENOTDIR:
                        shutil.copyfile(arg, hook_arg)
                    else:
                        log_error("Copying failed: %s" % exc)
                        exit_error()
        except IOError as e:
            log_error("Copying of hook script failed: %s" % e)
            exit_error()

    else:
        log_error("Unknown hook option '%s'" % deploy_name)
        exit_error()

load_pa_configuration()

shorten_envs()

os.chdir(VALUE_CURRENT_DIRECTORY)



# -*- coding: utf-8 -*-

from .views import *

from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required as lr

# these are private URLs for auth management

urlpatterns = patterns('',
    url(r'^list/$', lr(AuthListView.as_view()), name='auth-list'),
    url(r'^create/$', lr(CreateUserView.as_view()), name='auth-create'),
    url(r'^enable/$', lr(EnableAuthView.as_view()), name='auth-enable'),
    url(r'^disable/$', lr(DisableAuthView.as_view()), name='auth-disable'),
    url(r'^edit/(?P<username>[\w.@+-]+)/$', lr(EditUserView.as_view()), name='auth-edit'),
    url(r'^delete/(?P<username>[\w.@+-]+)/$', lr(DeleteAuthView.as_view()), name='auth-delete'),
)



# -*- coding: utf-8 -*-

from django.views.generic import FormView, TemplateView, View
from django.views.generic.list import ListView

from .forms import ResultForm

from .difference import TwoComparator


class TwoCompareView(FormView, TemplateView):
    template_name = "compare/two.html"

    def get(self, request, *args, **kwargs):
        context = {'title': 'Run comparison'}
        if request.GET:
            form = ResultForm(request.GET)
            if form.is_valid():
                diff = TwoComparator(form.cleaned_data['left_result'],
                                     form.cleaned_data['right_result'])
                context['diff'] = diff.compare()
                context['left'] = form.cleaned_data['left_result']
                context['right'] = form.cleaned_data['right_result']
        else:
            form = ResultForm()

        context['form'] = form
        return self.render_to_response(context)



# CONVENTION FOR WRITING UNIT TESTS
#  - The python file with unit tests shall be named with 'test_' prefix.
#  - Test methods need to be defined within a class derived from base.TestCase
#    (name of the class can be arbitrary).
#  - Test methods need to be named with 'test_' prefix.
#  - A unit test shall not modify/create/remove any file during its exectution.
#  - No unit test shall rely on any other unit test, i.e. each test shall be
#    possible to execute on its own.
#  - Prefer 'mock' decorator to setUp method. Use setUp only to avoid using
#    the same mock decorator for many unit test methods. See base.py for details
#    on the usage of mock.
#  - Each python file needs to have 'suite' function defined in order to be
#    executed along the other tests using 'python setup.py test'

# TEST EXECUTION
#  - Before publishing any new or updated test, make sure it's possible to
#    execute it on its own. Run:
#       nosetests <dir_to_python_modules>.<python_module>:<class>.<method>
#  - Example:
#       nosetests tests.test_example:TestExample.test_basic_example

# STANDARD OUTPUT OF A TEST
#  - To let the tests print to stdout, use nosetests option --nocapture

# Required import
try:
    import base
except ImportError:
    import tests.base as base

# Modules/symbols to be tested or mocked
import sys
from tests import test_preupg


class TestExample(base.TestCase):

    class list_mocked(list):
        pass

    class function_mocked(base.MockFunction):
        def __call__(self):
            self.var = sys.argv[0]

    @base.mock(sys, "argv", list_mocked)
    @base.mock(test_preupg, "suite", function_mocked())
    def test_basic_example(self):
        # Do some preparation before the test
        sys.argv = ["executable"]
        # Run the method/function to be tested
        test_preupg.suite()
        # Verify the results
        self.assertEqual(test_preupg.suite.var, "executable")



# -*- coding: utf-8 -*-

from __future__ import print_function, unicode_literals
import optparse
from optparse import OptionValueError

from preupg import settings
from preupg.version import VERSION


def upload_callback(option, dummy_opt_str, dummy_value, parser):
    if len(parser.rargs) == 0:
        setattr(parser.values, option.dest, True)
    else:
        if parser.rargs[0].startswith('-'):
            setattr(parser.values, option.dest, True)
        else:
            setattr(parser.values, option.dest, parser.rargs[0])
            try:
                second_arg = parser.rargs[1]
            except IndexError:
                pass
            else:
                if not second_arg.startswith('-'):
                    raise OptionValueError("Specify at most one argument for"
                                           " upload option.")


class CLI(object):

    """Class for processing data from commandline"""

    def __init__(self, args=None):
        """parse arguments"""
        self.parser = optparse.OptionParser(
            add_help_option=False,
            version="Preupgrade Assistant %s" % VERSION
        )

        self.add_args()
        if args:
            self.opts, self.args = self.parser.parse_args(args=args)
        else:
            self.opts, self.args = self.parser.parse_args()
        
        self.resolve_option_dependencies()

    def add_args(self):
        self.parser.add_option(
            "-h", "--help",
            action="help",
            help="Show help message and exit."
        )
        self.parser.add_option(
            "-S", "--skip-common",
            action="store_true",
            help="Skip generating files containing information about the"
                 " system. For assessing the system these files are needed"
                 " (by modules) but in the case the system remains the same"
                 " (the same installed packages, configuration files not"
                 " touched, etc.) they can be reused from the previous runs of"
                 " Preupgrade Assistant."
        )
        self.parser.add_option(
            "-d", "--debug",
            action="store_true",
            help="Turn on debugging mode."
        )
        self.parser.add_option(
            "-u", "--upload",
            dest="upload",
            metavar="URL",
            action="callback",
            callback=upload_callback,
            help="Upload a system assessment result to Preupgrade Assistant"
                 " WEB-UI."
        )
        self.parser.add_option(
            "-r", "--results",
            metavar="TARBALL",
            help="Provide path to a system assessment result tarball which is"
                 " to be uploaded to WEB-UI. By default, the result tarballs"
                 " can be found in %s." % settings.assessment_results_dir
        )
        self.parser.add_option(
            "-l", "--list-contents-set",
            action="store_true",
            help="List all the available sets of modules. They are searched"
                 " for in %s." % settings.source_dir
        )
        self.parser.add_option(
            "-s", "--scan",
            metavar="MODULE_SET",
            help="Provide name of the set of modules which are to be used for"
                 " assessing the system. By default, if there is just one set"
                 " in %s, Preupgrade Assistant uses that one. Use"
                 " --list-contents-set option to get a list of possible"
                 " values." % settings.source_dir
        )
        self.parser.add_option(
            "-c", "--contents",
            metavar="ALL_XCCDF_PATH",
            help="Provide path to all-xccdf.xml of the set of modules which is"
                 " to be used for assesing the system. By default, if there is"
                 " just one set in %s, Preupgrade Assistant uses that one."
                 " Option --scan works similarly." % settings.source_dir
        )
        self.parser.add_option(
            "--riskcheck",
            action="store_true",
            default=False,
            help="Return the highest reported level of risk or result related"
                 " to system upgrade. Run Preupgrade Assistant first -"
                 " assessment of the system needs to be performed before using"
                 " this option. When this option is used in concert with"
                 " --verbose option, summary of the risks are printed to"
                 " STDOUT. If the --verbose option is used once, just HIGH and"
                 " EXTREME risks are printed. If it is used twice, all the"
                 " risks are printed. " + " " * 30 +
                 "Return codes:" + " " * 45 +
                 "0 ... SLIGHT or MEDIUM risk or needs_inspection, fixed, "
                 "informational, not_applicable, not_selected, not_checked or"
                 " pass result." + " " * 50 +
                 "1 ... HIGH risk or needs_action result." + " " * 25 +
                 "2 ... EXTREME risk or error or fail result."
        )
        self.parser.add_option(
            "--force",
            action="store_true",
            default=False,
            help="Suppress user interaction."
        )
        self.parser.add_option(
            "--text",
            action="store_true",
            default=False,
            help="Generate plain text assessment report alongside XML and HTML"
                 " reports. The text report is converted from HTML using"
                 " elinks, lynx or w3m tool."
        )
        self.parser.add_option(
            "-v", "--verbose",
            action="count",
            default=0,
            help="Show more information during the assessment."
        )
        self.parser.add_option(
            "--cleanup",
            action="store_true",
            default=False,
            help="Remove all the files created by previous runs of Preupgrade"
                 " Assistant."
        )
        self.parser.add_option(
            "-m", "--mode",
            metavar="MODE",
            choices=['migrate', 'upgrade'],
            help="Select what you plan to do with the system after performing"
                 " its assessment by Preupgrate Assistant - migration or"
                 " upgrade. Both modes are selected by default. This option"
                 " may only affect behaviour of the modules - they can provide"
                 " different results when only one mode is selected. Use one"
                 " of these values: migrate, upgrade. It may be that modules"
                 " behave the same no matter what mode is selected."
        )
        self.parser.add_option(
            "--select-rules",
            metavar="RULES",
            help="Execute just a subset of modules out of a module set."
                 " Multiple modules are to be separated by a comma."
        )
        self.parser.add_option(
            "--list-rules",
            action="store_true",
            default=False,
            help="List all the modules available within a module set."
        )
        self.parser.add_option(
            "--dst-arch",
            metavar="ARCH",
            help=(
                "Specify an architecture of the system to be migrate to."
                " Available option are: %s. Use of the option is expected on"
                " 32-bit systems as by the release of RHEL 7, 32-bit hardware"
                " support has been dropped."
                % ", ".join(settings.migration_options)
            )
        )
        self.parser.add_option(
            "--old-report-style",
            action="store_true",
            help="Generate report with simpler style than the default."
        )

    def resolve_option_dependencies(self):
        if self.opts.scan and self.opts.contents:
            raise OptionValueError("Use either --scan or --contents option,"
                                   " not both.")


if __name__ == '__main__':
    x = CLI()
    print(x.args.id)



# -*- coding: utf-8 -*-

"""
Class creates a set of packages for migration scenario
"""

import os

from preupg.utils import FileHelper
from preupg.logger import *
from preupg.kickstart.application import BaseKickstart
from preupg import settings


class YumGroupManager(object):
    """more intelligent dict; enables searching in yum groups"""
    def __init__(self):
        self.groups = {}

    def add(self, group):
        self.groups[group.name] = group

    def find_match(self, packages):
        """is there a group whose packages are subset of argument 'packages'?"""
        groups = []
        for group in iter(self.groups.values()):
            if len(group.required) != 0:
                if group.match(packages):
                    groups.append(group)
        return groups

    def __str__(self):
        return "%s: %d groups" % (self.__class__.__name__, len(self.groups.values()))


class YumGroup(object):
    def __init__(self, name, mandatory, default, optional):
        self.name = name
        self.mandatory = mandatory
        self.mandatory_set = set(mandatory)
        self.default = default
        self.optional = optional
        self.required = set(mandatory + default)
        self.missing = []

    def __str__(self):
        return "%s (%d required packages)" % (self.name, len(self.required))

    def __repr__(self):
        return "<%s: M:%s D:%s O:%s>" % (self.name, self.mandatory, self.default, self.optional)

    def match(self, packages):
        found = self.required.issubset(packages)
        if self.name == 'core':
            return True
        if not found:
            if len(self.required) == 1:
                return False
            cnt_pkgs = 0
            for pkg in self.required:
                if pkg not in packages:
                    cnt_pkgs += 1
                self.missing.append(pkg)
            if len(self.required) == int(cnt_pkgs):
                return False
            found = True
        return found

    def exclude_mandatory(self, packages):
        # New set with elements in packages but not in self.required
        return packages.difference(self.required)

    def exclude_optional(self, packages):
        # New set with elements in packages but not in self.optional
        return packages.difference(self.optional)


class YumGroupGenerator(object):
    """class for aggregating packages into yum groups"""

    def __init__(self, package_list, removed_packages, dependency_list, *args, **kwargs):
        """
        we dont take info about groups from yum, but from dark matrix, format is:

        group_name | mandatory packages | default packages | optional

        package_list is a list of packages which should aggregated into groups
        args is a list of filepaths to files where group definitions are stored
        """
        self.packages = set(package_list)
        self.removed_packages = set(removed_packages)
        self.dependency_list = set(dependency_list)
        self.gm = YumGroupManager()
        self.group_def_fp = []
        for p in args:
            if os.path.exists(p):
                self.group_def_fp.append(p)
                self._read_group_info()

    def _read_group_info(self):
        def get_packages(s):
            # get rid of empty strings
            return [x for x in s.strip().split(',') if x]

        for fp in self.group_def_fp:
            lines = FileHelper.get_file_content(fp, 'r', True)
            for line in lines:
                stuff = line.split('|')
                name = stuff[0].strip()
                mandatory = get_packages(stuff[1])
                default = get_packages(stuff[2])
                optional = get_packages(stuff[3])
                # why would we want empty groups?
                if mandatory or default or optional:
                    yg = YumGroup(name, mandatory, default, optional)
                    self.gm.add(yg)

    def remove_packages(self, packages):
        for pkg in self.removed_packages:
            if pkg in packages:
                packages.remove(pkg)
        return packages

    def remove_dependencies(self, output_packages):
        # Remove dependencies from kickstart
        # New set with elements in output_packages but not in self.dependency_list
        return output_packages.difference(self.dependency_list)

    def get_list(self):
        groups = self.gm.find_match(self.packages)
        output_groups = []
        output_packages = self.packages
        missing_installed = []
        for group in groups:
            if len(group.required) != 0:
                output_groups.append('@'+group.name)
                if self.dependency_list:
                    missing = [x for x in group.missing_installed if x not in self.dependency_list]
                    missing_installed.extend([x + ' # group ' + group.name for x in missing])
                output_packages = group.exclude_mandatory(output_packages)
                output_packages = group.exclude_optional(output_packages)
        output_groups.sort()
        output_packages = list(self.remove_dependencies(output_packages))
        output_packages.sort()
        return output_groups, output_packages, missing_installed


class PackagesHandling(BaseKickstart):
    """class for replacing/updating package names"""

    def __init__(self, handler):
        """
        we dont take info about groups from yum, but from dark matrix, format is:

        group_name | mandatory packages | default packages | optional

        package_list is a list of packages which should aggregated into groups
        args is a list of filepaths to files where group definitions are stored
        """
        self.packages = None
        self.obsoleted = None
        self.handler = handler
        self.installed_dependencies = None
        self.special_pkg_list = None

    def replace_obsolete(self):
        # obsolete list has format like
        # old_pkg|required-by-pkg|obsoleted-by-pkgs|repo-id
        if self.packages:
            for cnt, pkgs in enumerate(self.packages):
                found = [x for x in self.obsoleted if pkgs in x]
                if found:
                    fields = found[0].split('|')
                    self.packages[cnt] = fields[2]

    @staticmethod
    def get_package_list(filename, field=None):
        """
        content packages/ReplacedPackages is taking care of packages, which were
        replaced/obsoleted/removed between releases. It produces a file with a list
        of packages which should be installed.
        """
        full_path_name = os.path.join(settings.KS_DIR, filename)
        if not os.path.exists(full_path_name):
            return []
        lines = FileHelper.get_file_content(full_path_name, 'rb', method=True, decode_flag=True)
        # Remove newline character from list
        package_list = []
        for line in lines:
            # We have to go over all lines and remove all commented.
            if line.startswith('#'):
                continue
            if field is None:
                package_list.append(line.strip())
            else:
                try:
                    # Format of file is like
                    # old-package|required-by-pkgs|replaced-by-pkgs|repoid
                    pkg_field = line.split('|')
                    if pkg_field[field] is not None:
                        package_list.append(pkg_field[field])
                except ValueError:
                    # Line seems to be wrong, go to the next one
                    pass
        return package_list

    @staticmethod
    def get_installed_packages():
        prefix = 'RHRHEL7rpmlist_'
        result_list = []
        list_files = ['kept', 'kept-notbase',
                      'replaced', 'replaced-notbase']
        for l in list_files:
            try:
                result_list.extend(PackagesHandling.get_package_list(prefix + l, 2))
            except IOError:
                log_message("The '%s' file was not found. Skipping the package generation.", (prefix + l))
                return None
        return result_list

    @staticmethod
    def get_installed_dependencies(obsoleted):
        dep_list = []
        deps = PackagesHandling.get_package_list('first_dependencies', field=None)
        for pkg in deps:
            pkg = pkg.strip()
            if pkg.startswith('/'):
                continue
            if '.so' in pkg:
                continue
            if '(' in pkg:
                dep_list.append(pkg.split('(')[0])
                continue
            found = [x for x in obsoleted if pkg in x]
            if found:
                fields = found[0].split('|')
                dep_list.append(fields[2])
            else:
                dep_list.append(pkg.split()[0])

        return dep_list

    def output_packages(self):
        """ outputs %packages section """
        self.packages = PackagesHandling.get_installed_packages()
        if self.packages is None:
            return None, None
        try:
            self.obsoleted = PackagesHandling.get_package_list('RHRHEL7rpmlist_obsoleted')
        except IOError:
            self.obsoleted = []
        try:
            self.special_pkg_list = PackagesHandling.get_package_list('special_pkg_list')
        except IOError:
            self.special_pkg_list = []

        self.installed_dependencies = PackagesHandling.get_installed_dependencies(self.obsoleted)
        self.installed_dependencies = list(set(self.installed_dependencies))
        self.installed_dependencies.sort()
        # remove files which are replaced by another package
        self.replace_obsolete()

        removed_packages = []
        remove_pkg_optional = os.path.join(settings.KS_DIR, 'RemovedPkg-optional')
        if os.path.exists(remove_pkg_optional):
            try:
                removed_packages = FileHelper.get_file_content(remove_pkg_optional, 'r', method=True)
            except IOError:
                return None, None
        # TODO We should think about if ObsoletedPkg-{required,optional} should be used
        if not removed_packages:
            return None, None
        abs_fps = [os.path.join(settings.KS_DIR, fp) for fp in settings.KS_FILES]
        ygg = YumGroupGenerator(self.packages, removed_packages, self.installed_dependencies, *abs_fps)
        groups, self.packages, missing_installed = ygg.get_list()
        self.packages = ygg.remove_packages(self.packages)
        return groups, missing_installed

    def run_module(self, *args, **kwargs):
        (groups, missing_installed) = self.output_packages()
        if self.packages or groups:
            if self.special_pkg_list:
                self.packages.extend(self.special_pkg_list)
            self.handler.packages.packageList = self.packages
            self.handler.packages.groupList = groups
            if missing_installed:
                self.handler.packages.excludedList = missing_installed




# -*- coding: utf-8 -*-

from __future__ import unicode_literals, print_function
import os
import re
try:
    import rpm
except ImportError:
    pass
from preupg.utils import FileHelper
from preupg import settings
from preupg.logger import logger_report


def html_escape(text):
    """
    HTML escape text; text is a list of strings
    """
    escapes = [
        ("&", "&amp;"),  # '&' has to be first!
        ("<", "&lt;"),
        (">", "&gt;"),
        ("'", "&apos;"),
        ('"', "&quot;"),
    ]
    for esc, html_esc in escapes:
        text = text.replace(esc, html_esc)
    return text


def link_update(value):
    """
    Function replaces [link:sss] with either
    <a href="http:sss">sss</a> or
    <a href="./sss">sss</a> for file link
    """
    possible_links = ['http', 'https', 'ftp']
    if [x for x in possible_links if x in value]:
        return '<a href="{0}">{0}</a>'.format(value.strip())
    else:
        if value.strip().startswith("/") or value.strip().startswith(".."):
            return value.strip()
        else:
            return '<a href="./{0}">{0}</a>'.format(value.strip())


def bold_update(value):
    """
    Function replaces [bold:sss] with <b>sss</b>
    """
    return '<b>{0}</b>'.format(value)


def tag_formating(text):
    """
    Format tags like:
    [bold: some text] -> <b>some text</b>
    [link: http://127.0.0.1]
        ->
            <a href="http://127.0.0.1">http://127.0.0.1</a>
    [link: /var/cache/description.txt]
        ->
            <a href="/var/cache/description.txt">/var/cache/description.txt</a>
    """
    def repl(match):
        if match.group("tag") == 'bold':
            return bold_update(match.group("text"))
        elif match.group("tag") == 'link':
            return link_update(match.group("text"))
        else:
            return match.group(0)  # return the original matched string

    regular = re.compile(r'\[(?P<tag>\w+):(?P<text>.+?)\]', re.MULTILINE)
    return re.sub(regular, repl, text)


def get_package_version(name):
    """
    Function return a package name and version
    """
    ts = rpm.TransactionSet()
    mi = ts.dbMatch()
    for h in mi:
        if h['name'] == name:
            return "%s-%s-%s" % (h['name'], h['version'], h['release'])


class XmlManager(object):
    """
    Class operates with XML oscap result
    """
    def __init__(self, assessment_result_path, copied_module_set_path):
        """
        assessment_result_path .. path to the directory where all results of
                                  the assessment are stored
        copied_module_set_path .. path to the module set directory copied for
                                 the assessment to the assessment_result_path
        """
        self.assessment_result_path = assessment_result_path
        self.copied_module_set_path = copied_module_set_path
        self.paths_to_all_modules = self.get_module_dirs()
        self.solution_texts = {}

    def update_report(self, report_path):
        """Update XML or HTML report with relevant solution texts."""
        if not self.solution_texts:
            self.load_solution_texts()

        orig_file = os.path.join(self.assessment_result_path, report_path)
        report_content = FileHelper.get_file_content(orig_file, "rb")

        for solution_placeholer, solution_text in self.solution_texts.items():
            report_content = report_content.replace(solution_placeholer,
                                                    solution_text)

        FileHelper.write_to_file(orig_file, "wb", report_content)

    def load_solution_texts(self):
        """Load solution texts into a dictionary."""
        for dir_name in self.paths_to_all_modules:
            section = dir_name.replace(
                self.copied_module_set_path, "").replace("/", "_")
            solution_placeholder = section + "_SOLUTION_MSG"

            logger_report.debug("Processing solution placeholder '%s'",
                                solution_placeholder)
            try:
                solution_text = FileHelper.get_file_content(
                    os.path.join(dir_name, settings.solution_txt), "rb")
            except IOError:
                # solution file is not mandatory
                solution_text = ""

            updated_solution_text = self.get_updated_solution(solution_text)
            self.solution_texts[solution_placeholder] = updated_solution_text

    def get_updated_solution(self, solution_text):
        """Function converts the solution text to HTML"""
        formatted_text = tag_formating(html_escape(solution_text))
        formatted_text = formatted_text.replace("\n", "<br/>\n")
        return formatted_text

    def get_module_dirs(self):
        """Find all directories that contain a module."""
        paths_to_all_modules = []
        for path, _, dir_files in os.walk(self.assessment_result_path):
            # Each module has its INI file
            if settings.check_script in dir_files:
                paths_to_all_modules.append(path)
        return paths_to_all_modules



# -*- coding: utf-8 -*-

"""
Class appends users and groups to kickstart
"""

import os

from preupg import settings
from preupg.utils import FileHelper
from preupg.kickstart.application import BaseKickstart


class UsersGroupsGenerator(BaseKickstart):

    """Generate users"""
    def __init__(self, handler):
        self.handler = handler
        self.user_perm = None
        self.group_perm = None
        self.groups = None

    @staticmethod
    def get_kickstart_users(filename, groups=None, splitter=":"):
        """
        returns dictionary with names and uid, gid, etc.
        :param filename: filename with Users in /root/preupgrade/kickstart directory
        :param groups: dictionary with groups
        :param splitter: delimiter for parsing files
        :return: dictionary with users
        """
        try:
            lines = FileHelper.get_file_content(os.path.join(settings.KS_DIR, filename), 'rb', method=True)
        except IOError:
            return None
        lines = [x for x in lines if not x.startswith('#') and not x.startswith(' ')]
        user_dict = {}
        for line in lines:
            fields = line.strip().split(splitter)
            try:
                user_group = []
                if groups:
                    for key, value in iter(groups.items()):
                        found = [x for x in iter(value.values()) if fields[0] in x]
                        if found:
                            user_group.append(key)
                user_dict[fields[0]] = {}
                user_dict[fields[0]] = {'homedir': fields[5],
                                        'shell': fields[6],
                                        'uid': int(fields[2]),
                                        'gid': int(fields[3]),
                                        'groups': user_group}
            except IndexError:
                pass
        return user_dict

    @staticmethod
    def get_kickstart_groups(filename, splitter=":"):
        """
        returns dictionary with names and uid, gid, etc.
        :param filename: filename with Users in /root/preupgrade/kickstart directory
        :param splitter: delimiter for parsing files
        :return: dictionary with users
        """
        try:
            lines = FileHelper.get_file_content(os.path.join(settings.KS_DIR, filename), 'rb', method=True)
        except IOError:
            return None
        lines = [x for x in lines if not x.startswith('#') and not x.startswith(' ')]
        group_dict = {}
        for line in lines:
            fields = line.split(splitter)
            try:
                group_dict[fields[0]] = {}
                group_dict[fields[0]] = {fields[2]: fields[3].strip().split(',')}
            except IndexError:
                pass
        return group_dict

    def collect_data(self):
        self.group_perm = UsersGroupsGenerator.get_kickstart_groups('Groups')
        self.user_perm = UsersGroupsGenerator.get_kickstart_users('Users', groups=self.group_perm)

    def update_users(self, users):
        if not users:
            return None
        for key, value in iter(users.items()):
            self.handler.user.dataList().append(self.handler.UserData(name=key,
                                                                      uid=int(value['uid']),
                                                                      gid=int(value['gid']),
                                                                      shell=value['shell'],
                                                                      homedir=value['homedir'],
                                                                      groups=value['groups']))

    def update_groups(self, groups):
        if not groups:
            return None
        for key, value in iter(groups.items()):
            for gid in iter(value.keys()):
                self.handler.group.dataList().append(self.handler.GroupData(name=key, gid=gid))

    def filter_kickstart_users(self):
        kickstart_users = {}
        if not self.user_perm:
            return None
        setup_passwd = UsersGroupsGenerator.get_kickstart_users('setup_passwd')
        uidgid = UsersGroupsGenerator.get_kickstart_users('uidgid', splitter='|')
        for user, ids in iter(self.user_perm.items()):
            if setup_passwd:
                if [x for x in iter(setup_passwd.keys()) if user in x]:
                    continue
            if uidgid:
                if [x for x in iter(uidgid.keys()) if user in x]:
                    continue
            kickstart_users[user] = ids
        if not kickstart_users:
            return None
        return kickstart_users

    def filter_kickstart_groups(self):
        kickstart_groups = {}
        if not self.group_perm:
            return None
        uidgid = UsersGroupsGenerator.get_kickstart_users('uidgid', splitter='|')
        for group, ids in iter(self.group_perm.items()):
            if uidgid:
                if [x for x in iter(uidgid.keys()) if group in x]:
                    continue
            kickstart_groups[group] = ids
        if not kickstart_groups:
            return None
        return kickstart_groups

    def run_module(self):
        self.collect_data()
        self.update_users(self.filter_kickstart_users())
        self.update_groups(self.filter_kickstart_groups())



# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required as lr
from .views import TwoCompareView

urlpatterns = patterns(
    '',
    url(r'^$', lr(TwoCompareView.as_view()), name='compare'),
    url(r'^two-compare/$', lr(TwoCompareView.as_view()), name='two-compare'),
)



# -*- coding: utf-8 -*-

class Enum(object):
    """Enumerated objects."""
    __slots__ = (
        "_items",
        "_help",
        '_display',
    )

    def __init__(self, items):
        """
        @param items: items to be enumerated
        @type items: dict or list of tuples
            {key: (human readable text, (helper text))}
            [(key, human readable text, (helper text)), (...)]
        """
        self._items = {}
        self._help = {}
        self._display = {}

        for key in items:
            try:
                value = items[key]
            except (IndexError, TypeError):
                try:
                    help_text = key[2]
                except IndexError:
                    pass
                try:
                    display_text = key[3]
                except IndexError:
                    pass
                value = key[1]
                key = key[0]

            if key in self._items:
                raise ValueError("Duplicite item: %s" % key)
            if isinstance(value, tuple):
                self._help[key] = value[1]
                self._items[key] = value[0]
            else:
                self._items[key] = value
                try:
                    self._help[key] = help_text
                except NameError:
                    pass
                try:
                    self._display[key] = display_text
                except NameError:
                    pass

    def __iter__(self):
        return iter(self._items)

    def __getitem__(self, key):
        return self._items[key]

    def get(self, key, default=None):
        try:
            return self[key]
        except (IndexError, KeyError):
            return default

    def get_help(self, key):
        return self._help[key]

    def display(self, key):
        return self._display[key]

    def get_mapping(self):
        """Return list of (key, value) mappings."""
        return self._items.items()

    def get_display_mapping(self):
        """Return list of (key, value) mappings with nicer values."""
        return zip(self._items.values(), self._display.values())

    def list_keys(self, values=None):
        """ [values] -> [keys] or -> all [keys] """
        if values is not None:
            return [self.get_key(value) for value in values]
        else:
            return self._items.keys()

    def get_key(self, value):
        return filter(lambda x: x[1] == value,
                      self._items.items())[0][0]


def test():
    assert Enum({'k': ('key', 'blablabla')})['k'] == 'key'
    assert Enum([('k', 'key', 'blablabla')])['k'] == 'key'
    assert Enum({'k': ('key', 'blablabla')}).get_help('k') == 'blablabla'
    assert Enum([('k', 'key', 'blablabla')]).get_help('k') == 'blablabla'
    assert Enum({'k': 'key'})['k'] == 'key'
    assert Enum({'k': 'key'}).get_key('key') == 'k'
    assert Enum([('k', 'key', 'long key', 'nice key')]).display('k') == 'nice key'

if __name__ == '__main__':
    test()



from __future__ import unicode_literals
import unittest
import tempfile
import shutil
import os
from glob import glob

from preupg.xmlgen.compose import XCCDFCompose, ComposeXML
from preupg.utils import FileHelper
from preupg import settings

try:
    import base
except ImportError:
    import tests.base as base

FOO_DIR = 'FOOBAR6_7'
FOO_RESULTS = FOO_DIR + settings.results_postfix


class TestContentGenerate(base.TestCase):
    dir_name = None
    result_dir = None

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp(prefix='preupgrade', dir='/tmp')
        self.dir_name = os.path.join(os.getcwd(), 'tests', FOO_DIR)
        self.result_dir = os.path.join(self.temp_dir, 'tests', FOO_RESULTS)
        shutil.copytree(self.dir_name, os.path.join(self.temp_dir, FOO_DIR))
        self.dir_name = os.path.join(self.temp_dir, FOO_DIR)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        for d, subd, file_name in os.walk(self.dir_name):
            group_xml = [x for x in file_name if x == 'group.xml']
            if group_xml:
                os.unlink(os.path.join(d, group_xml[0]))

    def test_compose(self):
        ComposeXML().collect_group_xmls(self.dir_name, self.dir_name)
        for subdir in glob(os.path.join(self.dir_name, "*/")):
            self.assertTrue(os.path.exists(os.path.join(subdir, 'group.xml')))
            self.assertFalse(os.path.exists(
                os.path.join(subdir, settings.all_xccdf_xml_filename)))


class TestGlobalContent(base.TestCase):
    temp_dir = None
    dir_name = None
    result_dir = None

    def setUp(self):
        self.temp_dir = tempfile.mkdtemp(prefix='preupgrade', dir='/tmp')
        self.dir_name = os.path.join(os.getcwd(), 'tests', FOO_DIR)
        self.result_dir = os.path.join(self.temp_dir, FOO_DIR + '-results')
        shutil.copytree(self.dir_name, os.path.join(self.temp_dir, FOO_DIR))
        self.data_dir_orig = settings.data_dir
        settings.data_dir = os.path.join(os.getcwd(), "data")

    def tearDown(self):
        shutil.rmtree(self.temp_dir)
        settings.data_dir = self.data_dir_orig

    def test_final_compose(self):
        dir_name = os.path.join(self.temp_dir, FOO_DIR)
        ComposeXML().collect_group_xmls(dir_name, dir_name)

        xccdf_compose = XCCDFCompose(os.path.join(self.temp_dir, FOO_DIR))
        xccdf_compose.generate_xml()
        all_xccdf = os.path.join(self.result_dir,
                                 settings.all_xccdf_xml_filename)
        self.assertTrue(os.path.exists(all_xccdf))
        dummy_lines = FileHelper.get_file_content(all_xccdf, 'rb')


def suite():
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    suite.addTest(loader.loadTestsFromTestCase(TestContentGenerate))
    suite.addTest(loader.loadTestsFromTestCase(TestGlobalContent))
    return suite

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=3).run(suite())



# -*- coding: utf-8 -*-

from django.conf.urls import patterns, url
from django.contrib.auth.decorators import login_required as lr

from .views import RunsView, ReportView, NewRunView, NewHostView, DeleteOlderView, \
    NewLocalRunView, ReportFilesView, RunView, DeleteRunView, ResultViewAjax

urlpatterns = patterns(
    '',
    url(r'^$', lr(RunsView.as_view()), name='index'),
    url(r'^$', lr(RunsView.as_view()), name='results-list'),
    url(r'^delete-older/$', lr(DeleteOlderView.as_view()), name='delete-older'),
    url(r'^(?P<result_id>\d+)/detail/$', lr(RunView.as_view()), name='result-detail'),
    #url(r'^run/(?P<run_id>\d+)/$', lr(RunView.as_view()), name='run'),
    # TODO: creating runs from UI is not done and ready for production
    #url(r'^new/$', lr(NewRunView.as_view()), name='new-run'),
    #url(r'^new-local-run/$', lr(NewLocalRunView.as_view()), name='new-local-run'),
    #url(r'^new-host/$', lr(NewHostView.as_view()), name='new-host'),
    url(r'^(?P<result_id>\d+)/report/$', lr(ReportView.as_view()),  name='show-report'),
    url(r'^(?P<result_id>\d+)/file/$', lr(ReportFilesView.as_view()),        name='show-file'),
    url(r'^(?P<result_id>\d+)/ajax/$', lr(ResultViewAjax.as_view()), name='show-result-ajax'),
    url(r'^(?P<result_id>\d+)/delete/$', lr(DeleteRunView.as_view()), name='result-delete'),
)



# -*- coding: utf-8 -*-

"""
Class creates a set of packages for migration scenario
"""

import os

from preupg.utils import FileHelper
from preupg.kickstart.application import BaseKickstart
from preupg import settings


class ReposHandling(BaseKickstart):
    """class for replacing/updating package names"""

    def __init__(self, handler):
        """
        """
        self.handler = handler
        self.repos = ReposHandling.get_kickstart_repo('available-repos')

    @staticmethod
    def get_kickstart_repo(filename):
        """
        returns dictionary with names and URLs
        :param filename: filename with available-repos
        :return: dictionary with enabled repolist
        """
        try:
            lines = FileHelper.get_file_content(
                os.path.join(settings.KS_DIR, filename), 'rb', method=True
            )
        except IOError:
            return None
        lines = [x for x in lines
                 if not x.startswith('#') and not x.startswith(' ')]
        if not lines:
            return None
        repo_dict = {}
        for line in lines:
            fields = line.split('=')
            repo_dict[fields[0]] = fields[2]
        return repo_dict

    def update_repositories(self):
        if self.repos:
            for key, value in iter(self.repos.items()):
                self.handler.repo.dataList().append(
                    self.handler.RepoData(name=key, baseurl=value.strip())
                )

    def run_module(self, *args, **kwargs):
        self.update_repositories()



# -*- coding: utf-8 -*-

import logging
import subprocess


logger = logging.getLogger('preup_ui')


def run_subprocess(cmd, log=False, shell=False, to_stdout=False):
    sp = subprocess.Popen(cmd,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT,
                          shell=shell)
    while sp.poll() == None:
        if log or to_stdout:
            out = sp.stdout.readline().strip().decode('utf8')
            if log:
                logger.info(out)
            if to_stdout:
                print out
    logger.info('Cmd: \'%s\' finished with exit code: %d', ' '.join(cmd), sp.returncode)
    return sp.returncode



"""
This class will ready the YAML file as INI file.
So no change is needed from maintainer point of view
"""

from __future__ import print_function, unicode_literals
import os
import codecs

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

from preupg.xmlgen.xml_utils import XmlUtils
from preupg.utils import FileHelper, ModuleSetUtils
try:
    from xml.etree import ElementTree
except ImportError:
    from elementtree import ElementTree
from preupg import settings

try:
    from xml.etree.ElementTree import ParseError
except ImportError:
    from xml.parsers.expat import ExpatError as ParseError


class OscapGroupXml(object):

    """Class creates a XML file for OpenSCAP"""

    def __init__(self, module_set_dir, dir_name):
        """
        @param {str} module_set_dir - directory where all modules are stored
        @param {str} dir_name - directory of specific module or module-set
            directory
        """
        self.module_set_dir = module_set_dir
        self.dirname = dir_name
        if dir_name.endswith('/'):
            self.main_dir = dir_name.split('/')[-3]
        else:
            self.main_dir = dir_name.split('/')[-2]
        self.lists = []
        self.loaded = {}
        self.filename = "group.xml"
        self.rule = []
        self.ret = {}

    def find_all_ini(self):
        """
        Find all ini files in the self.dirname path. Then read each ini file
        and save all options in 'preupgrade' section with their values to
        self.loded dict:
        self.loaded[ini_file_path] = {option1: value, option2: value, ...}
        """
        for dir_name in os.listdir(self.dirname):
            if dir_name.endswith(".ini"):
                self.lists.append(os.path.join(self.dirname, dir_name))
        for file_name in self.lists:
            if FileHelper.check_file(file_name, "r") is False:
                continue
            filehander = codecs.open(file_name, 'r', encoding=settings.defenc)
            config = configparser.ConfigParser()
            config.readfp(filehander)
            fields = {}
            section = 'preupgrade'
            for option in config.options(section):
                fields[option] = config.get(section, option)
            self.loaded[file_name] = fields

    def collect_group_xmls(self):
        """The functions is used for collecting all INI files into the one."""
        # load content without decoding to unicode - ElementTree requests this
        xml_path = os.path.join(self.dirname, "group.xml")
        try:
            self.ret[self.dirname] = ElementTree.parse(xml_path).getroot()
        except ParseError as par_err:
            raise ParseError(
                "Encountered a parse error in file %s.\nDetails: %s"
                % (os.path.join(self.dirname, "group.xml"), par_err))
        return self.ret

    def write_xml(self):
        """The function is used for storing a group.xml file"""
        self.find_all_ini()
        self.write_list_rules()
        xml_utils = XmlUtils(self.module_set_dir, self.dirname, self.loaded)
        self.rule = xml_utils.prepare_sections()
        file_name = os.path.join(self.dirname, "group.xml")
        try:
            FileHelper.write_to_file(file_name, "wb",
                                     ["%s" % item for item in self.rule])
        except IOError as ior:
            raise IOError('Problem with writing to file %s.\nDetails: %s'
                          % (file_name, ior.message))

    def write_profile_xml(self, target_tree):
        """The function stores all-xccdf.xml file into content directory"""
        file_name = os.path.join(self.dirname, "all-xccdf.xml")
        print('File which can be used by Preupgrade-Assistant is: %s'
              % file_name)
        try:
            # encoding must be set! otherwise ElementTree return non-ascii
            # characters as html entities instead, which are unsusable for us
            data = ElementTree.tostring(target_tree, "utf-8")
            FileHelper.write_to_file(file_name, "wb", data, False)
        except IOError as ioe:
            raise IOError('Problem with writing to file %s.\nDetails: %s'
                          % (file_name, ioe.message))

    def write_list_rules(self):
        module_path = self.dirname.replace(self.module_set_dir, '')
        rule_name = '_'.join(module_path.split(os.sep)[1:])
        file_list_rules = os.path.join(settings.UPGRADE_PATH,
                                       settings.file_list_rules)
        lines = []
        if os.path.exists(file_list_rules):
            lines = FileHelper.get_file_content(file_list_rules, "rb",
                                                method=True)

        # add rule only for modules (dir which contains module.ini)
        if os.path.isfile(os.path.join(self.dirname, settings.module_ini)):
            lines.append(settings.xccdf_tag + rule_name + '_check' + '\n')

        FileHelper.write_to_file(file_list_rules, "wb", lines)



# Create your views here.

from .forms import GenericStateFilterForm
from .models import AppSettings
from django.views.generic import FormView, TemplateView, RedirectView


class SettingsView(TemplateView):
    template_name = "config/settings.html"

    def get_context_data(self, **kwargs):

        current_state_setting = AppSettings.get_initial_state_filter()
        if current_state_setting:
            state_form = GenericStateFilterForm(initial={'state': current_state_setting})
        else:
            state_form = GenericStateFilterForm()
        context = {'state_form': state_form}
        return context

class SetStateSettingsView(FormView, RedirectView):

    def post(self, request, *args, **kwargs):
        gsff = GenericStateFilterForm(self.request.POST)

        if gsff.is_valid:
            states = gsff['state']

        AppSettings.set_initial_state_filter(states)


